/****************************************************************************
** Meta object code from reading C++ file 'shell.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../neovim-qt/src/gui/shell.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'shell.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8NeovimQt5ShellE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN8NeovimQt5ShellE = QtMocHelpers::stringData(
    "NeovimQt::Shell",
    "neovimTitleChanged",
    "",
    "title",
    "neovimBusyChanged",
    "neovimResized",
    "rows",
    "cols",
    "neovimAttachmentChanged",
    "neovimMaximized",
    "neovimForeground",
    "neovimOpacity",
    "neovimSuspend",
    "neovimFullScreen",
    "neovimFrameless",
    "neovimGuiCloseRequest",
    "status",
    "neovimIsUnsupported",
    "neovimShowContextMenu",
    "colorsChanged",
    "setGuiAdaptiveColorEnabled",
    "isEnabled",
    "setGuiAdaptiveFontEnabled",
    "setGuiAdaptiveStyle",
    "styleName",
    "showGuiAdaptiveStyleList",
    "handleNeovimNotification",
    "name",
    "QVariantList",
    "args",
    "resizeNeovim",
    "n_cols",
    "n_rows",
    "setGuiFont",
    "fdesc",
    "force",
    "reset",
    "setGuiFontWide",
    "updateGuiWindowState",
    "Qt::WindowStates",
    "state",
    "openFiles",
    "QList<QUrl>",
    "url",
    "neovimError",
    "NeovimConnector::NeovimError",
    "neovimExited",
    "neovimResizeFinished",
    "mouseClickReset",
    "mouseClickIncrement",
    "Qt::MouseButton",
    "bt",
    "init",
    "handleFontError",
    "msg",
    "updateWindowId",
    "updateClientInfo",
    "handleGinitError",
    "msgid",
    "fun",
    "QVariant",
    "err",
    "handleShimError",
    "handleGetBackgroundOption",
    "val",
    "screenChanged",
    "setAttached",
    "attached",
    "ensureVisible",
    "isNeovimBusy",
    "isNeovimAttached"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN8NeovimQt5ShellE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      42,   14, // methods
       2,  388, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      19,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    1,  266,    2, 0x06,    3 /* Public */,
       4,    1,  269,    2, 0x06,    5 /* Public */,
       5,    2,  272,    2, 0x06,    7 /* Public */,
       8,    1,  277,    2, 0x06,   10 /* Public */,
       9,    1,  280,    2, 0x06,   12 /* Public */,
      10,    0,  283,    2, 0x06,   14 /* Public */,
      11,    1,  284,    2, 0x06,   15 /* Public */,
      12,    0,  287,    2, 0x06,   17 /* Public */,
      13,    1,  288,    2, 0x06,   18 /* Public */,
      14,    1,  291,    2, 0x06,   20 /* Public */,
      15,    1,  294,    2, 0x06,   22 /* Public */,
      15,    0,  297,    2, 0x26,   24 /* Public | MethodCloned */,
      17,    0,  298,    2, 0x06,   25 /* Public */,
      18,    0,  299,    2, 0x06,   26 /* Public */,
      19,    0,  300,    2, 0x06,   27 /* Public */,
      20,    1,  301,    2, 0x06,   28 /* Public */,
      22,    1,  304,    2, 0x06,   30 /* Public */,
      23,    1,  307,    2, 0x06,   32 /* Public */,
      25,    0,  310,    2, 0x06,   34 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
      26,    2,  311,    2, 0x0a,   35 /* Public */,
      30,    1,  316,    2, 0x0a,   38 /* Public */,
      30,    2,  319,    2, 0x0a,   40 /* Public */,
      33,    3,  324,    2, 0x0a,   43 /* Public */,
      33,    2,  331,    2, 0x2a,   47 /* Public | MethodCloned */,
      37,    1,  336,    2, 0x0a,   50 /* Public */,
      38,    1,  339,    2, 0x0a,   52 /* Public */,
      41,    1,  342,    2, 0x0a,   54 /* Public */,
      44,    1,  345,    2, 0x09,   56 /* Protected */,
      46,    1,  348,    2, 0x09,   58 /* Protected */,
      47,    0,  351,    2, 0x09,   60 /* Protected */,
      48,    0,  352,    2, 0x09,   61 /* Protected */,
      49,    1,  353,    2, 0x09,   62 /* Protected */,
      52,    0,  356,    2, 0x09,   64 /* Protected */,
      53,    1,  357,    2, 0x09,   65 /* Protected */,
      55,    0,  360,    2, 0x09,   67 /* Protected */,
      56,    0,  361,    2, 0x09,   68 /* Protected */,
      57,    3,  362,    2, 0x09,   69 /* Protected */,
      62,    3,  369,    2, 0x09,   73 /* Protected */,
      63,    3,  376,    2, 0x09,   77 /* Protected */,
      65,    0,  383,    2, 0x09,   81 /* Protected */,
      66,    1,  384,    2, 0x08,   82 /* Private */,
      68,    0,  387,    2, 0x08,   84 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    6,    7,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QByteArray, 0x80000000 | 28,   27,   29,
    QMetaType::Void, QMetaType::QSize,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   31,   32,
    QMetaType::Bool, QMetaType::QString, QMetaType::Bool, QMetaType::Bool,   34,   35,   36,
    QMetaType::Bool, QMetaType::QString, QMetaType::Bool,   34,   35,
    QMetaType::Bool, QMetaType::QString,   34,
    QMetaType::Void, 0x80000000 | 39,   40,
    QMetaType::Void, 0x80000000 | 42,   43,
    QMetaType::Void, 0x80000000 | 45,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 50,   51,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   54,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UInt, QMetaType::ULongLong, 0x80000000 | 60,   58,   59,   61,
    QMetaType::Void, QMetaType::UInt, QMetaType::ULongLong, 0x80000000 | 60,   58,   59,   61,
    QMetaType::Void, QMetaType::UInt, QMetaType::ULongLong, 0x80000000 | 60,   58,   59,   64,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   67,
    QMetaType::Void,

 // properties: name, type, flags, notifyId, revision
      69, QMetaType::Bool, 0x00015001, uint(1), 0,
      70, QMetaType::Bool, 0x00015001, uint(3), 0,

       0        // eod
};

Q_CONSTINIT const QMetaObject NeovimQt::Shell::staticMetaObject = { {
    QMetaObject::SuperData::link<ShellWidget::staticMetaObject>(),
    qt_meta_stringdata_ZN8NeovimQt5ShellE.offsetsAndSizes,
    qt_meta_data_ZN8NeovimQt5ShellE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN8NeovimQt5ShellE_t,
        // property 'isNeovimBusy'
        QtPrivate::TypeAndForceComplete<bool, std::true_type>,
        // property 'isNeovimAttached'
        QtPrivate::TypeAndForceComplete<bool, std::true_type>,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<Shell, std::true_type>,
        // method 'neovimTitleChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'neovimBusyChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'neovimResized'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'neovimAttachmentChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'neovimMaximized'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'neovimForeground'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'neovimOpacity'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<double, std::false_type>,
        // method 'neovimSuspend'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'neovimFullScreen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'neovimFrameless'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'neovimGuiCloseRequest'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'neovimGuiCloseRequest'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'neovimIsUnsupported'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'neovimShowContextMenu'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'colorsChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setGuiAdaptiveColorEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'setGuiAdaptiveFontEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'setGuiAdaptiveStyle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'showGuiAdaptiveStyleList'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'handleNeovimNotification'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QByteArray &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariantList &, std::false_type>,
        // method 'resizeNeovim'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QSize &, std::false_type>,
        // method 'resizeNeovim'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'setGuiFont'
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'setGuiFont'
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'setGuiFontWide'
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'updateGuiWindowState'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Qt::WindowStates, std::false_type>,
        // method 'openFiles'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QList<QUrl>, std::false_type>,
        // method 'neovimError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<NeovimConnector::NeovimError, std::false_type>,
        // method 'neovimExited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'neovimResizeFinished'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'mouseClickReset'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'mouseClickIncrement'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<Qt::MouseButton, std::false_type>,
        // method 'init'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'handleFontError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'updateWindowId'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'updateClientInfo'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'handleGinitError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint32, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint64, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'handleShimError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint32, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint64, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'handleGetBackgroundOption'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint32, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint64, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'screenChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setAttached'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'ensureVisible'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void NeovimQt::Shell::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Shell *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->neovimTitleChanged((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->neovimBusyChanged((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->neovimResized((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->neovimAttachmentChanged((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->neovimMaximized((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->neovimForeground(); break;
        case 6: _t->neovimOpacity((*reinterpret_cast< std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->neovimSuspend(); break;
        case 8: _t->neovimFullScreen((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->neovimFrameless((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->neovimGuiCloseRequest((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->neovimGuiCloseRequest(); break;
        case 12: _t->neovimIsUnsupported(); break;
        case 13: _t->neovimShowContextMenu(); break;
        case 14: _t->colorsChanged(); break;
        case 15: _t->setGuiAdaptiveColorEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->setGuiAdaptiveFontEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->setGuiAdaptiveStyle((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->showGuiAdaptiveStyleList(); break;
        case 19: _t->handleNeovimNotification((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariantList>>(_a[2]))); break;
        case 20: _t->resizeNeovim((*reinterpret_cast< std::add_pointer_t<QSize>>(_a[1]))); break;
        case 21: _t->resizeNeovim((*reinterpret_cast< std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int>>(_a[2]))); break;
        case 22: { bool _r = _t->setGuiFont((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->setGuiFont((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->setGuiFontWide((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->updateGuiWindowState((*reinterpret_cast< std::add_pointer_t<Qt::WindowStates>>(_a[1]))); break;
        case 26: _t->openFiles((*reinterpret_cast< std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 27: _t->neovimError((*reinterpret_cast< std::add_pointer_t<NeovimConnector::NeovimError>>(_a[1]))); break;
        case 28: _t->neovimExited((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->neovimResizeFinished(); break;
        case 30: _t->mouseClickReset(); break;
        case 31: _t->mouseClickIncrement((*reinterpret_cast< std::add_pointer_t<Qt::MouseButton>>(_a[1]))); break;
        case 32: _t->init(); break;
        case 33: _t->handleFontError((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 34: _t->updateWindowId(); break;
        case 35: _t->updateClientInfo(); break;
        case 36: _t->handleGinitError((*reinterpret_cast< std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<quint64>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 37: _t->handleShimError((*reinterpret_cast< std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<quint64>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 38: _t->handleGetBackgroundOption((*reinterpret_cast< std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<quint64>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 39: _t->screenChanged(); break;
        case 40: _t->setAttached((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->ensureVisible(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QUrl> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (Shell::*)(const QString & );
            if (_q_method_type _q_method = &Shell::neovimTitleChanged; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(bool );
            if (_q_method_type _q_method = &Shell::neovimBusyChanged; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(int , int );
            if (_q_method_type _q_method = &Shell::neovimResized; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(bool );
            if (_q_method_type _q_method = &Shell::neovimAttachmentChanged; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(bool );
            if (_q_method_type _q_method = &Shell::neovimMaximized; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)();
            if (_q_method_type _q_method = &Shell::neovimForeground; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 5;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(double );
            if (_q_method_type _q_method = &Shell::neovimOpacity; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 6;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)();
            if (_q_method_type _q_method = &Shell::neovimSuspend; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 7;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(bool );
            if (_q_method_type _q_method = &Shell::neovimFullScreen; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 8;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(bool );
            if (_q_method_type _q_method = &Shell::neovimFrameless; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 9;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(int );
            if (_q_method_type _q_method = &Shell::neovimGuiCloseRequest; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 10;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)();
            if (_q_method_type _q_method = &Shell::neovimIsUnsupported; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 12;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)();
            if (_q_method_type _q_method = &Shell::neovimShowContextMenu; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 13;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)();
            if (_q_method_type _q_method = &Shell::colorsChanged; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 14;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(bool );
            if (_q_method_type _q_method = &Shell::setGuiAdaptiveColorEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 15;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(bool );
            if (_q_method_type _q_method = &Shell::setGuiAdaptiveFontEnabled; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 16;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)(const QString & );
            if (_q_method_type _q_method = &Shell::setGuiAdaptiveStyle; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 17;
                return;
            }
        }
        {
            using _q_method_type = void (Shell::*)();
            if (_q_method_type _q_method = &Shell::showGuiAdaptiveStyleList; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 18;
                return;
            }
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isNeovimBusy(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isNeovimAttached(); break;
        default: break;
        }
    }
}

const QMetaObject *NeovimQt::Shell::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NeovimQt::Shell::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN8NeovimQt5ShellE.stringdata0))
        return static_cast<void*>(this);
    return ShellWidget::qt_metacast(_clname);
}

int NeovimQt::Shell::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ShellWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void NeovimQt::Shell::neovimTitleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void NeovimQt::Shell::neovimBusyChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void NeovimQt::Shell::neovimResized(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void NeovimQt::Shell::neovimAttachmentChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void NeovimQt::Shell::neovimMaximized(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void NeovimQt::Shell::neovimForeground()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void NeovimQt::Shell::neovimOpacity(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void NeovimQt::Shell::neovimSuspend()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void NeovimQt::Shell::neovimFullScreen(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void NeovimQt::Shell::neovimFrameless(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void NeovimQt::Shell::neovimGuiCloseRequest(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 12
void NeovimQt::Shell::neovimIsUnsupported()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void NeovimQt::Shell::neovimShowContextMenu()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void NeovimQt::Shell::colorsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void NeovimQt::Shell::setGuiAdaptiveColorEnabled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void NeovimQt::Shell::setGuiAdaptiveFontEnabled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void NeovimQt::Shell::setGuiAdaptiveStyle(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void NeovimQt::Shell::showGuiAdaptiveStyleList()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}
namespace {
struct qt_meta_tag_ZN8NeovimQt19ShellRequestHandlerE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN8NeovimQt19ShellRequestHandlerE = QtMocHelpers::stringData(
    "NeovimQt::ShellRequestHandler"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN8NeovimQt19ShellRequestHandlerE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

Q_CONSTINIT const QMetaObject NeovimQt::ShellRequestHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ZN8NeovimQt19ShellRequestHandlerE.offsetsAndSizes,
    qt_meta_data_ZN8NeovimQt19ShellRequestHandlerE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN8NeovimQt19ShellRequestHandlerE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<ShellRequestHandler, std::true_type>
    >,
    nullptr
} };

void NeovimQt::ShellRequestHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShellRequestHandler *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *NeovimQt::ShellRequestHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NeovimQt::ShellRequestHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN8NeovimQt19ShellRequestHandlerE.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "MsgpackRequestHandler"))
        return static_cast< MsgpackRequestHandler*>(this);
    return QObject::qt_metacast(_clname);
}

int NeovimQt::ShellRequestHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
