/****************************************************************************
** Meta object code from reading C++ file 'msgpackiodevice.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../neovim-qt/src/msgpackiodevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'msgpackiodevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8NeovimQt15MsgpackIODeviceE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN8NeovimQt15MsgpackIODeviceE = QtMocHelpers::stringData(
    "NeovimQt::MsgpackIODevice",
    "error",
    "",
    "NeovimQt::MsgpackIODevice::MsgpackError",
    "notification",
    "name",
    "QVariantList",
    "args",
    "encodingChanged",
    "encoding",
    "aboutToClose",
    "setError",
    "err",
    "msg",
    "dataAvailable",
    "dataAvailableStdin",
    "dataAvailableFd",
    "fd",
    "requestTimeout",
    "id",
    "MsgpackError",
    "NoError",
    "InvalidDevice",
    "InvalidMsgpack"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN8NeovimQt15MsgpackIODeviceE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       1,   95, // properties
       1,  100, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    1,   68,    2, 0x06,    3 /* Public */,
       4,    2,   71,    2, 0x06,    5 /* Public */,
       8,    1,   76,    2, 0x06,    8 /* Public */,
      10,    0,   79,    2, 0x06,   10 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
      11,    2,   80,    2, 0x09,   11 /* Protected */,
      14,    0,   85,    2, 0x09,   14 /* Protected */,
      15,    1,   86,    2, 0x09,   15 /* Protected */,
      16,    1,   89,    2, 0x09,   17 /* Protected */,
      18,    1,   92,    2, 0x09,   19 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, QMetaType::QByteArray, 0x80000000 | 6,    5,    7,
    QMetaType::Void, QMetaType::QByteArray,    9,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,   12,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::UInt,   19,

 // properties: name, type, flags, notifyId, revision
       1, 0x80000000 | 20, 0x00015009, uint(0), 0,

 // enums: name, alias, flags, count, data
      20,   20, 0x0,    3,  105,

 // enum data: key, value
      21, uint(NeovimQt::MsgpackIODevice::NoError),
      22, uint(NeovimQt::MsgpackIODevice::InvalidDevice),
      23, uint(NeovimQt::MsgpackIODevice::InvalidMsgpack),

       0        // eod
};

Q_CONSTINIT const QMetaObject NeovimQt::MsgpackIODevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ZN8NeovimQt15MsgpackIODeviceE.offsetsAndSizes,
    qt_meta_data_ZN8NeovimQt15MsgpackIODeviceE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN8NeovimQt15MsgpackIODeviceE_t,
        // property 'error'
        QtPrivate::TypeAndForceComplete<MsgpackError, std::true_type>,
        // enum 'MsgpackError'
        QtPrivate::TypeAndForceComplete<MsgpackIODevice::MsgpackError, std::true_type>,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<MsgpackIODevice, std::true_type>,
        // method 'error'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackIODevice::MsgpackError, std::false_type>,
        // method 'notification'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QByteArray &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariantList &, std::false_type>,
        // method 'encodingChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QByteArray &, std::false_type>,
        // method 'aboutToClose'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackIODevice::MsgpackError, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'dataAvailable'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'dataAvailableStdin'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QByteArray &, std::false_type>,
        // method 'dataAvailableFd'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'requestTimeout'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint32, std::false_type>
    >,
    nullptr
} };

void NeovimQt::MsgpackIODevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MsgpackIODevice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->error((*reinterpret_cast< std::add_pointer_t<NeovimQt::MsgpackIODevice::MsgpackError>>(_a[1]))); break;
        case 1: _t->notification((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariantList>>(_a[2]))); break;
        case 2: _t->encodingChanged((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 3: _t->aboutToClose(); break;
        case 4: _t->setError((*reinterpret_cast< std::add_pointer_t<NeovimQt::MsgpackIODevice::MsgpackError>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->dataAvailable(); break;
        case 6: _t->dataAvailableStdin((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 7: _t->dataAvailableFd((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->requestTimeout((*reinterpret_cast< std::add_pointer_t<quint32>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NeovimQt::MsgpackIODevice::MsgpackError >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NeovimQt::MsgpackIODevice::MsgpackError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (MsgpackIODevice::*)(NeovimQt::MsgpackIODevice::MsgpackError );
            if (_q_method_type _q_method = &MsgpackIODevice::error; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (MsgpackIODevice::*)(const QByteArray & , const QVariantList & );
            if (_q_method_type _q_method = &MsgpackIODevice::notification; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (MsgpackIODevice::*)(const QByteArray & );
            if (_q_method_type _q_method = &MsgpackIODevice::encodingChanged; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (MsgpackIODevice::*)();
            if (_q_method_type _q_method = &MsgpackIODevice::aboutToClose; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< MsgpackError*>(_v) = _t->errorCause(); break;
        default: break;
        }
    }
}

const QMetaObject *NeovimQt::MsgpackIODevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NeovimQt::MsgpackIODevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN8NeovimQt15MsgpackIODeviceE.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int NeovimQt::MsgpackIODevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void NeovimQt::MsgpackIODevice::error(NeovimQt::MsgpackIODevice::MsgpackError _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void NeovimQt::MsgpackIODevice::notification(const QByteArray & _t1, const QVariantList & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void NeovimQt::MsgpackIODevice::encodingChanged(const QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void NeovimQt::MsgpackIODevice::aboutToClose()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
