/****************************************************************************
** Meta object code from reading C++ file 'neovimconnector.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../neovim-qt/src/neovimconnector.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'neovimconnector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8NeovimQt15NeovimConnectorE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN8NeovimQt15NeovimConnectorE = QtMocHelpers::stringData(
    "NeovimQt::NeovimConnector",
    "ready",
    "",
    "error",
    "NeovimQt::NeovimConnector::NeovimError",
    "processExited",
    "exitCode",
    "aboutToClose",
    "fatalTimeout",
    "discoverMetadata",
    "processError",
    "QProcess::ProcessError",
    "socketError",
    "msgpackError",
    "NeovimError",
    "NoError",
    "NoMetadata",
    "MetadataDescriptorError",
    "UnexpectedMsg",
    "APIMisMatch",
    "NoSuchMethod",
    "FailedToStart",
    "Crashed",
    "SocketError",
    "MsgpackError",
    "RuntimeMsgpackError"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN8NeovimQt15NeovimConnectorE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       1,   83, // properties
       1,   88, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,   68,    2, 0x06,    3 /* Public */,
       3,    1,   69,    2, 0x06,    4 /* Public */,
       5,    1,   72,    2, 0x06,    6 /* Public */,
       7,    0,   75,    2, 0x06,    8 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       8,    0,   76,    2, 0x0a,    9 /* Public */,
       9,    0,   77,    2, 0x09,   10 /* Protected */,
      10,    1,   78,    2, 0x09,   11 /* Protected */,
      12,    0,   81,    2, 0x09,   13 /* Protected */,
      13,    0,   82,    2, 0x09,   14 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags, notifyId, revision
       1, QMetaType::Bool, 0x00015001, uint(0), 0,

 // enums: name, alias, flags, count, data
      14,   14, 0x0,   11,   93,

 // enum data: key, value
      15, uint(NeovimQt::NeovimConnector::NoError),
      16, uint(NeovimQt::NeovimConnector::NoMetadata),
      17, uint(NeovimQt::NeovimConnector::MetadataDescriptorError),
      18, uint(NeovimQt::NeovimConnector::UnexpectedMsg),
      19, uint(NeovimQt::NeovimConnector::APIMisMatch),
      20, uint(NeovimQt::NeovimConnector::NoSuchMethod),
      21, uint(NeovimQt::NeovimConnector::FailedToStart),
      22, uint(NeovimQt::NeovimConnector::Crashed),
      23, uint(NeovimQt::NeovimConnector::SocketError),
      24, uint(NeovimQt::NeovimConnector::MsgpackError),
      25, uint(NeovimQt::NeovimConnector::RuntimeMsgpackError),

       0        // eod
};

Q_CONSTINIT const QMetaObject NeovimQt::NeovimConnector::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ZN8NeovimQt15NeovimConnectorE.offsetsAndSizes,
    qt_meta_data_ZN8NeovimQt15NeovimConnectorE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN8NeovimQt15NeovimConnectorE_t,
        // property 'ready'
        QtPrivate::TypeAndForceComplete<bool, std::true_type>,
        // enum 'NeovimError'
        QtPrivate::TypeAndForceComplete<NeovimConnector::NeovimError, std::true_type>,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<NeovimConnector, std::true_type>,
        // method 'ready'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'error'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<NeovimQt::NeovimConnector::NeovimError, std::false_type>,
        // method 'processExited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'aboutToClose'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'fatalTimeout'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'discoverMetadata'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'processError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QProcess::ProcessError, std::false_type>,
        // method 'socketError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'msgpackError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void NeovimQt::NeovimConnector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NeovimConnector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ready(); break;
        case 1: _t->error((*reinterpret_cast< std::add_pointer_t<NeovimQt::NeovimConnector::NeovimError>>(_a[1]))); break;
        case 2: _t->processExited((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->aboutToClose(); break;
        case 4: _t->fatalTimeout(); break;
        case 5: _t->discoverMetadata(); break;
        case 6: _t->processError((*reinterpret_cast< std::add_pointer_t<QProcess::ProcessError>>(_a[1]))); break;
        case 7: _t->socketError(); break;
        case 8: _t->msgpackError(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NeovimQt::NeovimConnector::NeovimError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (NeovimConnector::*)();
            if (_q_method_type _q_method = &NeovimConnector::ready; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimConnector::*)(NeovimQt::NeovimConnector::NeovimError );
            if (_q_method_type _q_method = &NeovimConnector::error; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimConnector::*)(int );
            if (_q_method_type _q_method = &NeovimConnector::processExited; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimConnector::*)();
            if (_q_method_type _q_method = &NeovimConnector::aboutToClose; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isReady(); break;
        default: break;
        }
    }
}

const QMetaObject *NeovimQt::NeovimConnector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NeovimQt::NeovimConnector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN8NeovimQt15NeovimConnectorE.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int NeovimQt::NeovimConnector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void NeovimQt::NeovimConnector::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void NeovimQt::NeovimConnector::error(NeovimQt::NeovimConnector::NeovimError _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void NeovimQt::NeovimConnector::processExited(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void NeovimQt::NeovimConnector::aboutToClose()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
