/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../neovim-qt/src/gui/mainwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8NeovimQt10MainWindowE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN8NeovimQt10MainWindowE = QtMocHelpers::stringData(
    "NeovimQt::MainWindow",
    "neovimAttachmentChanged",
    "",
    "closing",
    "activeChanged",
    "NeovimQt::MainWindow&",
    "window",
    "neovimSetTitle",
    "title",
    "neovimWidgetResized",
    "neovimMaximized",
    "neovimForeground",
    "neovimSuspend",
    "neovimFullScreen",
    "neovimFrameless",
    "neovimGuiCloseRequest",
    "neovimExited",
    "status",
    "neovimError",
    "NeovimConnector::NeovimError",
    "reconnectNeovim",
    "handleNeovimAttachment",
    "neovimIsUnsupported",
    "saveWindowGeometry",
    "setGuiAdaptiveColorEnabled",
    "isEnabled",
    "setGuiAdaptiveFontEnabled",
    "setGuiAdaptiveStyle",
    "style",
    "showGuiAdaptiveStyleList"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN8NeovimQt10MainWindowE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    1,  140,    2, 0x06,    1 /* Public */,
       3,    1,  143,    2, 0x06,    3 /* Public */,
       4,    1,  146,    2, 0x06,    5 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       7,    1,  149,    2, 0x08,    7 /* Private */,
       9,    0,  152,    2, 0x08,    9 /* Private */,
      10,    1,  153,    2, 0x08,   10 /* Private */,
      11,    0,  156,    2, 0x08,   12 /* Private */,
      12,    0,  157,    2, 0x08,   13 /* Private */,
      13,    1,  158,    2, 0x08,   14 /* Private */,
      14,    1,  161,    2, 0x08,   16 /* Private */,
      15,    1,  164,    2, 0x08,   18 /* Private */,
      16,    1,  167,    2, 0x08,   20 /* Private */,
      18,    1,  170,    2, 0x08,   22 /* Private */,
      20,    0,  173,    2, 0x08,   24 /* Private */,
      21,    1,  174,    2, 0x08,   25 /* Private */,
      22,    0,  177,    2, 0x08,   27 /* Private */,
      23,    0,  178,    2, 0x08,   28 /* Private */,
      24,    1,  179,    2, 0x08,   29 /* Private */,
      26,    1,  182,    2, 0x08,   31 /* Private */,
      27,    1,  185,    2, 0x08,   33 /* Private */,
      29,    0,  188,    2, 0x08,   35 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 5,    6,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   25,
    QMetaType::Void, QMetaType::Bool,   25,
    QMetaType::Void, QMetaType::QString,   28,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject NeovimQt::MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_ZN8NeovimQt10MainWindowE.offsetsAndSizes,
    qt_meta_data_ZN8NeovimQt10MainWindowE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN8NeovimQt10MainWindowE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<MainWindow, std::true_type>,
        // method 'neovimAttachmentChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'closing'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'activeChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<NeovimQt::MainWindow &, std::false_type>,
        // method 'neovimSetTitle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'neovimWidgetResized'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'neovimMaximized'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'neovimForeground'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'neovimSuspend'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'neovimFullScreen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'neovimFrameless'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'neovimGuiCloseRequest'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'neovimExited'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'neovimError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<NeovimConnector::NeovimError, std::false_type>,
        // method 'reconnectNeovim'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'handleNeovimAttachment'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'neovimIsUnsupported'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveWindowGeometry'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'setGuiAdaptiveColorEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'setGuiAdaptiveFontEnabled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'setGuiAdaptiveStyle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'showGuiAdaptiveStyleList'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void NeovimQt::MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->neovimAttachmentChanged((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->closing((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->activeChanged((*reinterpret_cast< std::add_pointer_t<NeovimQt::MainWindow&>>(_a[1]))); break;
        case 3: _t->neovimSetTitle((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->neovimWidgetResized(); break;
        case 5: _t->neovimMaximized((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->neovimForeground(); break;
        case 7: _t->neovimSuspend(); break;
        case 8: _t->neovimFullScreen((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->neovimFrameless((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->neovimGuiCloseRequest((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->neovimExited((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->neovimError((*reinterpret_cast< std::add_pointer_t<NeovimConnector::NeovimError>>(_a[1]))); break;
        case 13: _t->reconnectNeovim(); break;
        case 14: _t->handleNeovimAttachment((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->neovimIsUnsupported(); break;
        case 16: _t->saveWindowGeometry(); break;
        case 17: _t->setGuiAdaptiveColorEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->setGuiAdaptiveFontEnabled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->setGuiAdaptiveStyle((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->showGuiAdaptiveStyleList(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (MainWindow::*)(bool );
            if (_q_method_type _q_method = &MainWindow::neovimAttachmentChanged; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (MainWindow::*)(int );
            if (_q_method_type _q_method = &MainWindow::closing; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (MainWindow::*)(NeovimQt::MainWindow & );
            if (_q_method_type _q_method = &MainWindow::activeChanged; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
    }
}

const QMetaObject *NeovimQt::MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NeovimQt::MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN8NeovimQt10MainWindowE.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int NeovimQt::MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void NeovimQt::MainWindow::neovimAttachmentChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void NeovimQt::MainWindow::closing(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void NeovimQt::MainWindow::activeChanged(NeovimQt::MainWindow & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
