/****************************************************************************
** Meta object code from reading C++ file 'neovimapi2.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../neovim-qt/src/auto/neovimapi2.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'neovimapi2.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8NeovimQt10NeovimApi2E_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN8NeovimQt10NeovimApi2E = QtMocHelpers::stringData(
    "NeovimQt::NeovimApi2",
    "error",
    "",
    "errmsg",
    "QVariant",
    "errObj",
    "neovimNotification",
    "name",
    "QVariantList",
    "args",
    "on_nvim_buf_line_count",
    "int64_t",
    "err_nvim_buf_line_count",
    "on_buffer_get_line",
    "err_buffer_get_line",
    "on_buffer_set_line",
    "err_buffer_set_line",
    "on_buffer_del_line",
    "err_buffer_del_line",
    "on_buffer_get_line_slice",
    "err_buffer_get_line_slice",
    "on_nvim_buf_get_lines",
    "err_nvim_buf_get_lines",
    "on_buffer_set_line_slice",
    "err_buffer_set_line_slice",
    "on_nvim_buf_set_lines",
    "err_nvim_buf_set_lines",
    "on_nvim_buf_get_var",
    "err_nvim_buf_get_var",
    "on_nvim_buf_get_changedtick",
    "err_nvim_buf_get_changedtick",
    "on_nvim_buf_set_var",
    "err_nvim_buf_set_var",
    "on_nvim_buf_del_var",
    "err_nvim_buf_del_var",
    "on_buffer_set_var",
    "err_buffer_set_var",
    "on_buffer_del_var",
    "err_buffer_del_var",
    "on_nvim_buf_get_option",
    "err_nvim_buf_get_option",
    "on_nvim_buf_set_option",
    "err_nvim_buf_set_option",
    "on_nvim_buf_get_number",
    "err_nvim_buf_get_number",
    "on_nvim_buf_get_name",
    "err_nvim_buf_get_name",
    "on_nvim_buf_set_name",
    "err_nvim_buf_set_name",
    "on_nvim_buf_is_valid",
    "err_nvim_buf_is_valid",
    "on_buffer_insert",
    "err_buffer_insert",
    "on_nvim_buf_get_mark",
    "err_nvim_buf_get_mark",
    "on_nvim_buf_add_highlight",
    "err_nvim_buf_add_highlight",
    "on_nvim_buf_clear_highlight",
    "err_nvim_buf_clear_highlight",
    "on_nvim_tabpage_list_wins",
    "QList<int64_t>",
    "err_nvim_tabpage_list_wins",
    "on_nvim_tabpage_get_var",
    "err_nvim_tabpage_get_var",
    "on_nvim_tabpage_set_var",
    "err_nvim_tabpage_set_var",
    "on_nvim_tabpage_del_var",
    "err_nvim_tabpage_del_var",
    "on_tabpage_set_var",
    "err_tabpage_set_var",
    "on_tabpage_del_var",
    "err_tabpage_del_var",
    "on_nvim_tabpage_get_win",
    "err_nvim_tabpage_get_win",
    "on_nvim_tabpage_get_number",
    "err_nvim_tabpage_get_number",
    "on_nvim_tabpage_is_valid",
    "err_nvim_tabpage_is_valid",
    "on_nvim_ui_attach",
    "err_nvim_ui_attach",
    "on_ui_attach",
    "err_ui_attach",
    "on_nvim_ui_detach",
    "err_nvim_ui_detach",
    "on_nvim_ui_try_resize",
    "err_nvim_ui_try_resize",
    "on_nvim_ui_set_option",
    "err_nvim_ui_set_option",
    "on_nvim_command",
    "err_nvim_command",
    "on_nvim_feedkeys",
    "err_nvim_feedkeys",
    "on_nvim_input",
    "err_nvim_input",
    "on_nvim_replace_termcodes",
    "err_nvim_replace_termcodes",
    "on_nvim_command_output",
    "err_nvim_command_output",
    "on_nvim_eval",
    "err_nvim_eval",
    "on_nvim_call_function",
    "err_nvim_call_function",
    "on_nvim_strwidth",
    "err_nvim_strwidth",
    "on_nvim_list_runtime_paths",
    "err_nvim_list_runtime_paths",
    "on_nvim_set_current_dir",
    "err_nvim_set_current_dir",
    "on_nvim_get_current_line",
    "err_nvim_get_current_line",
    "on_nvim_set_current_line",
    "err_nvim_set_current_line",
    "on_nvim_del_current_line",
    "err_nvim_del_current_line",
    "on_nvim_get_var",
    "err_nvim_get_var",
    "on_nvim_set_var",
    "err_nvim_set_var",
    "on_nvim_del_var",
    "err_nvim_del_var",
    "on_vim_set_var",
    "err_vim_set_var",
    "on_vim_del_var",
    "err_vim_del_var",
    "on_nvim_get_vvar",
    "err_nvim_get_vvar",
    "on_nvim_get_option",
    "err_nvim_get_option",
    "on_nvim_set_option",
    "err_nvim_set_option",
    "on_nvim_out_write",
    "err_nvim_out_write",
    "on_nvim_err_write",
    "err_nvim_err_write",
    "on_nvim_err_writeln",
    "err_nvim_err_writeln",
    "on_nvim_list_bufs",
    "err_nvim_list_bufs",
    "on_nvim_get_current_buf",
    "err_nvim_get_current_buf",
    "on_nvim_set_current_buf",
    "err_nvim_set_current_buf",
    "on_nvim_list_wins",
    "err_nvim_list_wins",
    "on_nvim_get_current_win",
    "err_nvim_get_current_win",
    "on_nvim_set_current_win",
    "err_nvim_set_current_win",
    "on_nvim_list_tabpages",
    "err_nvim_list_tabpages",
    "on_nvim_get_current_tabpage",
    "err_nvim_get_current_tabpage",
    "on_nvim_set_current_tabpage",
    "err_nvim_set_current_tabpage",
    "on_nvim_subscribe",
    "err_nvim_subscribe",
    "on_nvim_unsubscribe",
    "err_nvim_unsubscribe",
    "on_nvim_get_color_by_name",
    "err_nvim_get_color_by_name",
    "on_nvim_get_color_map",
    "QVariantMap",
    "err_nvim_get_color_map",
    "on_nvim_get_mode",
    "err_nvim_get_mode",
    "on_nvim_get_api_info",
    "err_nvim_get_api_info",
    "on_nvim_call_atomic",
    "err_nvim_call_atomic",
    "on_nvim_win_get_buf",
    "err_nvim_win_get_buf",
    "on_nvim_win_get_cursor",
    "err_nvim_win_get_cursor",
    "on_nvim_win_set_cursor",
    "err_nvim_win_set_cursor",
    "on_nvim_win_get_height",
    "err_nvim_win_get_height",
    "on_nvim_win_set_height",
    "err_nvim_win_set_height",
    "on_nvim_win_get_width",
    "err_nvim_win_get_width",
    "on_nvim_win_set_width",
    "err_nvim_win_set_width",
    "on_nvim_win_get_var",
    "err_nvim_win_get_var",
    "on_nvim_win_set_var",
    "err_nvim_win_set_var",
    "on_nvim_win_del_var",
    "err_nvim_win_del_var",
    "on_window_set_var",
    "err_window_set_var",
    "on_window_del_var",
    "err_window_del_var",
    "on_nvim_win_get_option",
    "err_nvim_win_get_option",
    "on_nvim_win_set_option",
    "err_nvim_win_set_option",
    "on_nvim_win_get_position",
    "err_nvim_win_get_position",
    "on_nvim_win_get_tabpage",
    "err_nvim_win_get_tabpage",
    "on_nvim_win_get_number",
    "err_nvim_win_get_number",
    "on_nvim_win_is_valid",
    "err_nvim_win_is_valid",
    "on_buffer_line_count",
    "err_buffer_line_count",
    "on_buffer_get_lines",
    "err_buffer_get_lines",
    "on_buffer_set_lines",
    "err_buffer_set_lines",
    "on_buffer_get_var",
    "err_buffer_get_var",
    "on_buffer_get_option",
    "err_buffer_get_option",
    "on_buffer_set_option",
    "err_buffer_set_option",
    "on_buffer_get_number",
    "err_buffer_get_number",
    "on_buffer_get_name",
    "err_buffer_get_name",
    "on_buffer_set_name",
    "err_buffer_set_name",
    "on_buffer_is_valid",
    "err_buffer_is_valid",
    "on_buffer_get_mark",
    "err_buffer_get_mark",
    "on_buffer_add_highlight",
    "err_buffer_add_highlight",
    "on_buffer_clear_highlight",
    "err_buffer_clear_highlight",
    "on_tabpage_get_windows",
    "err_tabpage_get_windows",
    "on_tabpage_get_var",
    "err_tabpage_get_var",
    "on_tabpage_get_window",
    "err_tabpage_get_window",
    "on_tabpage_is_valid",
    "err_tabpage_is_valid",
    "on_ui_detach",
    "err_ui_detach",
    "on_ui_try_resize",
    "err_ui_try_resize",
    "on_vim_command",
    "err_vim_command",
    "on_vim_feedkeys",
    "err_vim_feedkeys",
    "on_vim_input",
    "err_vim_input",
    "on_vim_replace_termcodes",
    "err_vim_replace_termcodes",
    "on_vim_command_output",
    "err_vim_command_output",
    "on_vim_eval",
    "err_vim_eval",
    "on_vim_call_function",
    "err_vim_call_function",
    "on_vim_strwidth",
    "err_vim_strwidth",
    "on_vim_list_runtime_paths",
    "err_vim_list_runtime_paths",
    "on_vim_change_directory",
    "err_vim_change_directory",
    "on_vim_get_current_line",
    "err_vim_get_current_line",
    "on_vim_set_current_line",
    "err_vim_set_current_line",
    "on_vim_del_current_line",
    "err_vim_del_current_line",
    "on_vim_get_var",
    "err_vim_get_var",
    "on_vim_get_vvar",
    "err_vim_get_vvar",
    "on_vim_get_option",
    "err_vim_get_option",
    "on_vim_set_option",
    "err_vim_set_option",
    "on_vim_out_write",
    "err_vim_out_write",
    "on_vim_err_write",
    "err_vim_err_write",
    "on_vim_report_error",
    "err_vim_report_error",
    "on_vim_get_buffers",
    "err_vim_get_buffers",
    "on_vim_get_current_buffer",
    "err_vim_get_current_buffer",
    "on_vim_set_current_buffer",
    "err_vim_set_current_buffer",
    "on_vim_get_windows",
    "err_vim_get_windows",
    "on_vim_get_current_window",
    "err_vim_get_current_window",
    "on_vim_set_current_window",
    "err_vim_set_current_window",
    "on_vim_get_tabpages",
    "err_vim_get_tabpages",
    "on_vim_get_current_tabpage",
    "err_vim_get_current_tabpage",
    "on_vim_set_current_tabpage",
    "err_vim_set_current_tabpage",
    "on_vim_subscribe",
    "err_vim_subscribe",
    "on_vim_unsubscribe",
    "err_vim_unsubscribe",
    "on_vim_name_to_color",
    "err_vim_name_to_color",
    "on_vim_get_color_map",
    "err_vim_get_color_map",
    "on_window_get_buffer",
    "err_window_get_buffer",
    "on_window_get_cursor",
    "err_window_get_cursor",
    "on_window_set_cursor",
    "err_window_set_cursor",
    "on_window_get_height",
    "err_window_get_height",
    "on_window_set_height",
    "err_window_set_height",
    "on_window_get_width",
    "err_window_get_width",
    "on_window_set_width",
    "err_window_set_width",
    "on_window_get_var",
    "err_window_get_var",
    "on_window_get_option",
    "err_window_get_option",
    "on_window_set_option",
    "err_window_set_option",
    "on_window_get_position",
    "err_window_get_position",
    "on_window_get_tabpage",
    "err_window_get_tabpage",
    "on_window_is_valid",
    "err_window_is_valid",
    "handleResponse",
    "id",
    "fun",
    "handleResponseError",
    "nvim_buf_line_count",
    "NeovimQt::MsgpackRequest*",
    "buffer",
    "buffer_get_line",
    "index",
    "buffer_set_line",
    "line",
    "buffer_del_line",
    "buffer_get_line_slice",
    "start",
    "end",
    "include_start",
    "include_end",
    "nvim_buf_get_lines",
    "strict_indexing",
    "buffer_set_line_slice",
    "replacement",
    "nvim_buf_set_lines",
    "nvim_buf_get_var",
    "nvim_buf_get_changedtick",
    "nvim_buf_set_var",
    "value",
    "nvim_buf_del_var",
    "buffer_set_var",
    "buffer_del_var",
    "nvim_buf_get_option",
    "nvim_buf_set_option",
    "nvim_buf_get_number",
    "nvim_buf_get_name",
    "nvim_buf_set_name",
    "nvim_buf_is_valid",
    "buffer_insert",
    "lnum",
    "lines",
    "nvim_buf_get_mark",
    "nvim_buf_add_highlight",
    "src_id",
    "hl_group",
    "col_start",
    "col_end",
    "nvim_buf_clear_highlight",
    "line_start",
    "line_end",
    "nvim_tabpage_list_wins",
    "tabpage",
    "nvim_tabpage_get_var",
    "nvim_tabpage_set_var",
    "nvim_tabpage_del_var",
    "tabpage_set_var",
    "tabpage_del_var",
    "nvim_tabpage_get_win",
    "nvim_tabpage_get_number",
    "nvim_tabpage_is_valid",
    "nvim_ui_attach",
    "width",
    "height",
    "options",
    "ui_attach",
    "enable_rgb",
    "nvim_ui_detach",
    "nvim_ui_try_resize",
    "nvim_ui_set_option",
    "nvim_command",
    "command",
    "nvim_feedkeys",
    "keys",
    "mode",
    "escape_csi",
    "nvim_input",
    "nvim_replace_termcodes",
    "str",
    "from_part",
    "do_lt",
    "special",
    "nvim_command_output",
    "nvim_eval",
    "expr",
    "nvim_call_function",
    "fname",
    "nvim_strwidth",
    "nvim_list_runtime_paths",
    "nvim_set_current_dir",
    "dir",
    "nvim_get_current_line",
    "nvim_set_current_line",
    "nvim_del_current_line",
    "nvim_get_var",
    "nvim_set_var",
    "nvim_del_var",
    "vim_set_var",
    "vim_del_var",
    "nvim_get_vvar",
    "nvim_get_option",
    "nvim_set_option",
    "nvim_out_write",
    "nvim_err_write",
    "nvim_err_writeln",
    "nvim_list_bufs",
    "nvim_get_current_buf",
    "nvim_set_current_buf",
    "nvim_list_wins",
    "nvim_get_current_win",
    "nvim_set_current_win",
    "window",
    "nvim_list_tabpages",
    "nvim_get_current_tabpage",
    "nvim_set_current_tabpage",
    "nvim_subscribe",
    "event",
    "nvim_unsubscribe",
    "nvim_get_color_by_name",
    "nvim_get_color_map",
    "nvim_get_mode",
    "nvim_get_api_info",
    "nvim_call_atomic",
    "calls",
    "nvim_win_get_buf",
    "nvim_win_get_cursor",
    "nvim_win_set_cursor",
    "pos",
    "nvim_win_get_height",
    "nvim_win_set_height",
    "nvim_win_get_width",
    "nvim_win_set_width",
    "nvim_win_get_var",
    "nvim_win_set_var",
    "nvim_win_del_var",
    "window_set_var",
    "window_del_var",
    "nvim_win_get_option",
    "nvim_win_set_option",
    "nvim_win_get_position",
    "nvim_win_get_tabpage",
    "nvim_win_get_number",
    "nvim_win_is_valid",
    "buffer_line_count",
    "buffer_get_lines",
    "buffer_set_lines",
    "buffer_get_var",
    "buffer_get_option",
    "buffer_set_option",
    "buffer_get_number",
    "buffer_get_name",
    "buffer_set_name",
    "buffer_is_valid",
    "buffer_get_mark",
    "buffer_add_highlight",
    "buffer_clear_highlight",
    "tabpage_get_windows",
    "tabpage_get_var",
    "tabpage_get_window",
    "tabpage_is_valid",
    "ui_detach",
    "ui_try_resize",
    "vim_command",
    "vim_feedkeys",
    "vim_input",
    "vim_replace_termcodes",
    "vim_command_output",
    "vim_eval",
    "vim_call_function",
    "vim_strwidth",
    "vim_list_runtime_paths",
    "vim_change_directory",
    "vim_get_current_line",
    "vim_set_current_line",
    "vim_del_current_line",
    "vim_get_var",
    "vim_get_vvar",
    "vim_get_option",
    "vim_set_option",
    "vim_out_write",
    "vim_err_write",
    "vim_report_error",
    "vim_get_buffers",
    "vim_get_current_buffer",
    "vim_set_current_buffer",
    "vim_get_windows",
    "vim_get_current_window",
    "vim_set_current_window",
    "vim_get_tabpages",
    "vim_get_current_tabpage",
    "vim_set_current_tabpage",
    "vim_subscribe",
    "vim_unsubscribe",
    "vim_name_to_color",
    "vim_get_color_map",
    "window_get_buffer",
    "window_get_cursor",
    "window_set_cursor",
    "window_get_height",
    "window_set_height",
    "window_get_width",
    "window_set_width",
    "window_get_var",
    "window_get_option",
    "window_set_option",
    "window_get_position",
    "window_get_tabpage",
    "window_is_valid",
    "FunctionId",
    "NEOVIM_FN_NULL",
    "NEOVIM_FN_NVIM_BUF_LINE_COUNT",
    "NEOVIM_FN_BUFFER_GET_LINE",
    "NEOVIM_FN_BUFFER_SET_LINE",
    "NEOVIM_FN_BUFFER_DEL_LINE",
    "NEOVIM_FN_BUFFER_GET_LINE_SLICE",
    "NEOVIM_FN_NVIM_BUF_GET_LINES",
    "NEOVIM_FN_BUFFER_SET_LINE_SLICE",
    "NEOVIM_FN_NVIM_BUF_SET_LINES",
    "NEOVIM_FN_NVIM_BUF_GET_VAR",
    "NEOVIM_FN_NVIM_BUF_GET_CHANGEDTICK",
    "NEOVIM_FN_NVIM_BUF_SET_VAR",
    "NEOVIM_FN_NVIM_BUF_DEL_VAR",
    "NEOVIM_FN_BUFFER_SET_VAR",
    "NEOVIM_FN_BUFFER_DEL_VAR",
    "NEOVIM_FN_NVIM_BUF_GET_OPTION",
    "NEOVIM_FN_NVIM_BUF_SET_OPTION",
    "NEOVIM_FN_NVIM_BUF_GET_NUMBER",
    "NEOVIM_FN_NVIM_BUF_GET_NAME",
    "NEOVIM_FN_NVIM_BUF_SET_NAME",
    "NEOVIM_FN_NVIM_BUF_IS_VALID",
    "NEOVIM_FN_BUFFER_INSERT",
    "NEOVIM_FN_NVIM_BUF_GET_MARK",
    "NEOVIM_FN_NVIM_BUF_ADD_HIGHLIGHT",
    "NEOVIM_FN_NVIM_BUF_CLEAR_HIGHLIGHT",
    "NEOVIM_FN_NVIM_TABPAGE_LIST_WINS",
    "NEOVIM_FN_NVIM_TABPAGE_GET_VAR",
    "NEOVIM_FN_NVIM_TABPAGE_SET_VAR",
    "NEOVIM_FN_NVIM_TABPAGE_DEL_VAR",
    "NEOVIM_FN_TABPAGE_SET_VAR",
    "NEOVIM_FN_TABPAGE_DEL_VAR",
    "NEOVIM_FN_NVIM_TABPAGE_GET_WIN",
    "NEOVIM_FN_NVIM_TABPAGE_GET_NUMBER",
    "NEOVIM_FN_NVIM_TABPAGE_IS_VALID",
    "NEOVIM_FN_NVIM_UI_ATTACH",
    "NEOVIM_FN_UI_ATTACH",
    "NEOVIM_FN_NVIM_UI_DETACH",
    "NEOVIM_FN_NVIM_UI_TRY_RESIZE",
    "NEOVIM_FN_NVIM_UI_SET_OPTION",
    "NEOVIM_FN_NVIM_COMMAND",
    "NEOVIM_FN_NVIM_FEEDKEYS",
    "NEOVIM_FN_NVIM_INPUT",
    "NEOVIM_FN_NVIM_REPLACE_TERMCODES",
    "NEOVIM_FN_NVIM_COMMAND_OUTPUT",
    "NEOVIM_FN_NVIM_EVAL",
    "NEOVIM_FN_NVIM_CALL_FUNCTION",
    "NEOVIM_FN_NVIM_STRWIDTH",
    "NEOVIM_FN_NVIM_LIST_RUNTIME_PATHS",
    "NEOVIM_FN_NVIM_SET_CURRENT_DIR",
    "NEOVIM_FN_NVIM_GET_CURRENT_LINE",
    "NEOVIM_FN_NVIM_SET_CURRENT_LINE",
    "NEOVIM_FN_NVIM_DEL_CURRENT_LINE",
    "NEOVIM_FN_NVIM_GET_VAR",
    "NEOVIM_FN_NVIM_SET_VAR",
    "NEOVIM_FN_NVIM_DEL_VAR",
    "NEOVIM_FN_VIM_SET_VAR",
    "NEOVIM_FN_VIM_DEL_VAR",
    "NEOVIM_FN_NVIM_GET_VVAR",
    "NEOVIM_FN_NVIM_GET_OPTION",
    "NEOVIM_FN_NVIM_SET_OPTION",
    "NEOVIM_FN_NVIM_OUT_WRITE",
    "NEOVIM_FN_NVIM_ERR_WRITE",
    "NEOVIM_FN_NVIM_ERR_WRITELN",
    "NEOVIM_FN_NVIM_LIST_BUFS",
    "NEOVIM_FN_NVIM_GET_CURRENT_BUF",
    "NEOVIM_FN_NVIM_SET_CURRENT_BUF",
    "NEOVIM_FN_NVIM_LIST_WINS",
    "NEOVIM_FN_NVIM_GET_CURRENT_WIN",
    "NEOVIM_FN_NVIM_SET_CURRENT_WIN",
    "NEOVIM_FN_NVIM_LIST_TABPAGES",
    "NEOVIM_FN_NVIM_GET_CURRENT_TABPAGE",
    "NEOVIM_FN_NVIM_SET_CURRENT_TABPAGE",
    "NEOVIM_FN_NVIM_SUBSCRIBE",
    "NEOVIM_FN_NVIM_UNSUBSCRIBE",
    "NEOVIM_FN_NVIM_GET_COLOR_BY_NAME",
    "NEOVIM_FN_NVIM_GET_COLOR_MAP",
    "NEOVIM_FN_NVIM_GET_MODE",
    "NEOVIM_FN_NVIM_GET_API_INFO",
    "NEOVIM_FN_NVIM_CALL_ATOMIC",
    "NEOVIM_FN_NVIM_WIN_GET_BUF",
    "NEOVIM_FN_NVIM_WIN_GET_CURSOR",
    "NEOVIM_FN_NVIM_WIN_SET_CURSOR",
    "NEOVIM_FN_NVIM_WIN_GET_HEIGHT",
    "NEOVIM_FN_NVIM_WIN_SET_HEIGHT",
    "NEOVIM_FN_NVIM_WIN_GET_WIDTH",
    "NEOVIM_FN_NVIM_WIN_SET_WIDTH",
    "NEOVIM_FN_NVIM_WIN_GET_VAR",
    "NEOVIM_FN_NVIM_WIN_SET_VAR",
    "NEOVIM_FN_NVIM_WIN_DEL_VAR",
    "NEOVIM_FN_WINDOW_SET_VAR",
    "NEOVIM_FN_WINDOW_DEL_VAR",
    "NEOVIM_FN_NVIM_WIN_GET_OPTION",
    "NEOVIM_FN_NVIM_WIN_SET_OPTION",
    "NEOVIM_FN_NVIM_WIN_GET_POSITION",
    "NEOVIM_FN_NVIM_WIN_GET_TABPAGE",
    "NEOVIM_FN_NVIM_WIN_GET_NUMBER",
    "NEOVIM_FN_NVIM_WIN_IS_VALID",
    "NEOVIM_FN_BUFFER_LINE_COUNT",
    "NEOVIM_FN_BUFFER_GET_LINES",
    "NEOVIM_FN_BUFFER_SET_LINES",
    "NEOVIM_FN_BUFFER_GET_VAR",
    "NEOVIM_FN_BUFFER_GET_OPTION",
    "NEOVIM_FN_BUFFER_SET_OPTION",
    "NEOVIM_FN_BUFFER_GET_NUMBER",
    "NEOVIM_FN_BUFFER_GET_NAME",
    "NEOVIM_FN_BUFFER_SET_NAME",
    "NEOVIM_FN_BUFFER_IS_VALID",
    "NEOVIM_FN_BUFFER_GET_MARK",
    "NEOVIM_FN_BUFFER_ADD_HIGHLIGHT",
    "NEOVIM_FN_BUFFER_CLEAR_HIGHLIGHT",
    "NEOVIM_FN_TABPAGE_GET_WINDOWS",
    "NEOVIM_FN_TABPAGE_GET_VAR",
    "NEOVIM_FN_TABPAGE_GET_WINDOW",
    "NEOVIM_FN_TABPAGE_IS_VALID",
    "NEOVIM_FN_UI_DETACH",
    "NEOVIM_FN_UI_TRY_RESIZE",
    "NEOVIM_FN_VIM_COMMAND",
    "NEOVIM_FN_VIM_FEEDKEYS",
    "NEOVIM_FN_VIM_INPUT",
    "NEOVIM_FN_VIM_REPLACE_TERMCODES",
    "NEOVIM_FN_VIM_COMMAND_OUTPUT",
    "NEOVIM_FN_VIM_EVAL",
    "NEOVIM_FN_VIM_CALL_FUNCTION",
    "NEOVIM_FN_VIM_STRWIDTH",
    "NEOVIM_FN_VIM_LIST_RUNTIME_PATHS",
    "NEOVIM_FN_VIM_CHANGE_DIRECTORY",
    "NEOVIM_FN_VIM_GET_CURRENT_LINE",
    "NEOVIM_FN_VIM_SET_CURRENT_LINE",
    "NEOVIM_FN_VIM_DEL_CURRENT_LINE",
    "NEOVIM_FN_VIM_GET_VAR",
    "NEOVIM_FN_VIM_GET_VVAR",
    "NEOVIM_FN_VIM_GET_OPTION",
    "NEOVIM_FN_VIM_SET_OPTION",
    "NEOVIM_FN_VIM_OUT_WRITE",
    "NEOVIM_FN_VIM_ERR_WRITE",
    "NEOVIM_FN_VIM_REPORT_ERROR",
    "NEOVIM_FN_VIM_GET_BUFFERS",
    "NEOVIM_FN_VIM_GET_CURRENT_BUFFER",
    "NEOVIM_FN_VIM_SET_CURRENT_BUFFER",
    "NEOVIM_FN_VIM_GET_WINDOWS",
    "NEOVIM_FN_VIM_GET_CURRENT_WINDOW",
    "NEOVIM_FN_VIM_SET_CURRENT_WINDOW",
    "NEOVIM_FN_VIM_GET_TABPAGES",
    "NEOVIM_FN_VIM_GET_CURRENT_TABPAGE",
    "NEOVIM_FN_VIM_SET_CURRENT_TABPAGE",
    "NEOVIM_FN_VIM_SUBSCRIBE",
    "NEOVIM_FN_VIM_UNSUBSCRIBE",
    "NEOVIM_FN_VIM_NAME_TO_COLOR",
    "NEOVIM_FN_VIM_GET_COLOR_MAP",
    "NEOVIM_FN_WINDOW_GET_BUFFER",
    "NEOVIM_FN_WINDOW_GET_CURSOR",
    "NEOVIM_FN_WINDOW_SET_CURSOR",
    "NEOVIM_FN_WINDOW_GET_HEIGHT",
    "NEOVIM_FN_WINDOW_SET_HEIGHT",
    "NEOVIM_FN_WINDOW_GET_WIDTH",
    "NEOVIM_FN_WINDOW_SET_WIDTH",
    "NEOVIM_FN_WINDOW_GET_VAR",
    "NEOVIM_FN_WINDOW_GET_OPTION",
    "NEOVIM_FN_WINDOW_SET_OPTION",
    "NEOVIM_FN_WINDOW_GET_POSITION",
    "NEOVIM_FN_WINDOW_GET_TABPAGE",
    "NEOVIM_FN_WINDOW_IS_VALID"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN8NeovimQt10NeovimApi2E[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
     487,   14, // methods
       0,    0, // properties
       1, 4787, // enums/sets
       0,    0, // constructors
       0,       // flags
     324,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    2, 2936,    2, 0x06,    2 /* Public */,
       6,    2, 2941,    2, 0x06,    5 /* Public */,
      10,    1, 2946,    2, 0x06,    8 /* Public */,
      12,    2, 2949,    2, 0x06,   10 /* Public */,
      13,    1, 2954,    2, 0x06,   13 /* Public */,
      14,    2, 2957,    2, 0x06,   15 /* Public */,
      15,    0, 2962,    2, 0x06,   18 /* Public */,
      16,    2, 2963,    2, 0x06,   19 /* Public */,
      17,    0, 2968,    2, 0x06,   22 /* Public */,
      18,    2, 2969,    2, 0x06,   23 /* Public */,
      19,    1, 2974,    2, 0x06,   26 /* Public */,
      20,    2, 2977,    2, 0x06,   28 /* Public */,
      21,    1, 2982,    2, 0x06,   31 /* Public */,
      22,    2, 2985,    2, 0x06,   33 /* Public */,
      23,    0, 2990,    2, 0x06,   36 /* Public */,
      24,    2, 2991,    2, 0x06,   37 /* Public */,
      25,    0, 2996,    2, 0x06,   40 /* Public */,
      26,    2, 2997,    2, 0x06,   41 /* Public */,
      27,    1, 3002,    2, 0x06,   44 /* Public */,
      28,    2, 3005,    2, 0x06,   46 /* Public */,
      29,    1, 3010,    2, 0x06,   49 /* Public */,
      30,    2, 3013,    2, 0x06,   51 /* Public */,
      31,    0, 3018,    2, 0x06,   54 /* Public */,
      32,    2, 3019,    2, 0x06,   55 /* Public */,
      33,    0, 3024,    2, 0x06,   58 /* Public */,
      34,    2, 3025,    2, 0x06,   59 /* Public */,
      35,    1, 3030,    2, 0x06,   62 /* Public */,
      36,    2, 3033,    2, 0x06,   64 /* Public */,
      37,    1, 3038,    2, 0x06,   67 /* Public */,
      38,    2, 3041,    2, 0x06,   69 /* Public */,
      39,    1, 3046,    2, 0x06,   72 /* Public */,
      40,    2, 3049,    2, 0x06,   74 /* Public */,
      41,    0, 3054,    2, 0x06,   77 /* Public */,
      42,    2, 3055,    2, 0x06,   78 /* Public */,
      43,    1, 3060,    2, 0x06,   81 /* Public */,
      44,    2, 3063,    2, 0x06,   83 /* Public */,
      45,    1, 3068,    2, 0x06,   86 /* Public */,
      46,    2, 3071,    2, 0x06,   88 /* Public */,
      47,    0, 3076,    2, 0x06,   91 /* Public */,
      48,    2, 3077,    2, 0x06,   92 /* Public */,
      49,    1, 3082,    2, 0x06,   95 /* Public */,
      50,    2, 3085,    2, 0x06,   97 /* Public */,
      51,    0, 3090,    2, 0x06,  100 /* Public */,
      52,    2, 3091,    2, 0x06,  101 /* Public */,
      53,    1, 3096,    2, 0x06,  104 /* Public */,
      54,    2, 3099,    2, 0x06,  106 /* Public */,
      55,    1, 3104,    2, 0x06,  109 /* Public */,
      56,    2, 3107,    2, 0x06,  111 /* Public */,
      57,    0, 3112,    2, 0x06,  114 /* Public */,
      58,    2, 3113,    2, 0x06,  115 /* Public */,
      59,    1, 3118,    2, 0x06,  118 /* Public */,
      61,    2, 3121,    2, 0x06,  120 /* Public */,
      62,    1, 3126,    2, 0x06,  123 /* Public */,
      63,    2, 3129,    2, 0x06,  125 /* Public */,
      64,    0, 3134,    2, 0x06,  128 /* Public */,
      65,    2, 3135,    2, 0x06,  129 /* Public */,
      66,    0, 3140,    2, 0x06,  132 /* Public */,
      67,    2, 3141,    2, 0x06,  133 /* Public */,
      68,    1, 3146,    2, 0x06,  136 /* Public */,
      69,    2, 3149,    2, 0x06,  138 /* Public */,
      70,    1, 3154,    2, 0x06,  141 /* Public */,
      71,    2, 3157,    2, 0x06,  143 /* Public */,
      72,    1, 3162,    2, 0x06,  146 /* Public */,
      73,    2, 3165,    2, 0x06,  148 /* Public */,
      74,    1, 3170,    2, 0x06,  151 /* Public */,
      75,    2, 3173,    2, 0x06,  153 /* Public */,
      76,    1, 3178,    2, 0x06,  156 /* Public */,
      77,    2, 3181,    2, 0x06,  158 /* Public */,
      78,    0, 3186,    2, 0x06,  161 /* Public */,
      79,    2, 3187,    2, 0x06,  162 /* Public */,
      80,    0, 3192,    2, 0x06,  165 /* Public */,
      81,    2, 3193,    2, 0x06,  166 /* Public */,
      82,    0, 3198,    2, 0x06,  169 /* Public */,
      83,    2, 3199,    2, 0x06,  170 /* Public */,
      84,    0, 3204,    2, 0x06,  173 /* Public */,
      85,    2, 3205,    2, 0x06,  174 /* Public */,
      86,    0, 3210,    2, 0x06,  177 /* Public */,
      87,    2, 3211,    2, 0x06,  178 /* Public */,
      88,    0, 3216,    2, 0x06,  181 /* Public */,
      89,    2, 3217,    2, 0x06,  182 /* Public */,
      90,    0, 3222,    2, 0x06,  185 /* Public */,
      91,    2, 3223,    2, 0x06,  186 /* Public */,
      92,    1, 3228,    2, 0x06,  189 /* Public */,
      93,    2, 3231,    2, 0x06,  191 /* Public */,
      94,    1, 3236,    2, 0x06,  194 /* Public */,
      95,    2, 3239,    2, 0x06,  196 /* Public */,
      96,    1, 3244,    2, 0x06,  199 /* Public */,
      97,    2, 3247,    2, 0x06,  201 /* Public */,
      98,    1, 3252,    2, 0x06,  204 /* Public */,
      99,    2, 3255,    2, 0x06,  206 /* Public */,
     100,    1, 3260,    2, 0x06,  209 /* Public */,
     101,    2, 3263,    2, 0x06,  211 /* Public */,
     102,    1, 3268,    2, 0x06,  214 /* Public */,
     103,    2, 3271,    2, 0x06,  216 /* Public */,
     104,    1, 3276,    2, 0x06,  219 /* Public */,
     105,    2, 3279,    2, 0x06,  221 /* Public */,
     106,    0, 3284,    2, 0x06,  224 /* Public */,
     107,    2, 3285,    2, 0x06,  225 /* Public */,
     108,    1, 3290,    2, 0x06,  228 /* Public */,
     109,    2, 3293,    2, 0x06,  230 /* Public */,
     110,    0, 3298,    2, 0x06,  233 /* Public */,
     111,    2, 3299,    2, 0x06,  234 /* Public */,
     112,    0, 3304,    2, 0x06,  237 /* Public */,
     113,    2, 3305,    2, 0x06,  238 /* Public */,
     114,    1, 3310,    2, 0x06,  241 /* Public */,
     115,    2, 3313,    2, 0x06,  243 /* Public */,
     116,    0, 3318,    2, 0x06,  246 /* Public */,
     117,    2, 3319,    2, 0x06,  247 /* Public */,
     118,    0, 3324,    2, 0x06,  250 /* Public */,
     119,    2, 3325,    2, 0x06,  251 /* Public */,
     120,    1, 3330,    2, 0x06,  254 /* Public */,
     121,    2, 3333,    2, 0x06,  256 /* Public */,
     122,    1, 3338,    2, 0x06,  259 /* Public */,
     123,    2, 3341,    2, 0x06,  261 /* Public */,
     124,    1, 3346,    2, 0x06,  264 /* Public */,
     125,    2, 3349,    2, 0x06,  266 /* Public */,
     126,    1, 3354,    2, 0x06,  269 /* Public */,
     127,    2, 3357,    2, 0x06,  271 /* Public */,
     128,    0, 3362,    2, 0x06,  274 /* Public */,
     129,    2, 3363,    2, 0x06,  275 /* Public */,
     130,    0, 3368,    2, 0x06,  278 /* Public */,
     131,    2, 3369,    2, 0x06,  279 /* Public */,
     132,    0, 3374,    2, 0x06,  282 /* Public */,
     133,    2, 3375,    2, 0x06,  283 /* Public */,
     134,    0, 3380,    2, 0x06,  286 /* Public */,
     135,    2, 3381,    2, 0x06,  287 /* Public */,
     136,    1, 3386,    2, 0x06,  290 /* Public */,
     137,    2, 3389,    2, 0x06,  292 /* Public */,
     138,    1, 3394,    2, 0x06,  295 /* Public */,
     139,    2, 3397,    2, 0x06,  297 /* Public */,
     140,    0, 3402,    2, 0x06,  300 /* Public */,
     141,    2, 3403,    2, 0x06,  301 /* Public */,
     142,    1, 3408,    2, 0x06,  304 /* Public */,
     143,    2, 3411,    2, 0x06,  306 /* Public */,
     144,    1, 3416,    2, 0x06,  309 /* Public */,
     145,    2, 3419,    2, 0x06,  311 /* Public */,
     146,    0, 3424,    2, 0x06,  314 /* Public */,
     147,    2, 3425,    2, 0x06,  315 /* Public */,
     148,    1, 3430,    2, 0x06,  318 /* Public */,
     149,    2, 3433,    2, 0x06,  320 /* Public */,
     150,    1, 3438,    2, 0x06,  323 /* Public */,
     151,    2, 3441,    2, 0x06,  325 /* Public */,
     152,    0, 3446,    2, 0x06,  328 /* Public */,
     153,    2, 3447,    2, 0x06,  329 /* Public */,
     154,    0, 3452,    2, 0x06,  332 /* Public */,
     155,    2, 3453,    2, 0x06,  333 /* Public */,
     156,    0, 3458,    2, 0x06,  336 /* Public */,
     157,    2, 3459,    2, 0x06,  337 /* Public */,
     158,    1, 3464,    2, 0x06,  340 /* Public */,
     159,    2, 3467,    2, 0x06,  342 /* Public */,
     160,    1, 3472,    2, 0x06,  345 /* Public */,
     162,    2, 3475,    2, 0x06,  347 /* Public */,
     163,    1, 3480,    2, 0x06,  350 /* Public */,
     164,    2, 3483,    2, 0x06,  352 /* Public */,
     165,    1, 3488,    2, 0x06,  355 /* Public */,
     166,    2, 3491,    2, 0x06,  357 /* Public */,
     167,    1, 3496,    2, 0x06,  360 /* Public */,
     168,    2, 3499,    2, 0x06,  362 /* Public */,
     169,    1, 3504,    2, 0x06,  365 /* Public */,
     170,    2, 3507,    2, 0x06,  367 /* Public */,
     171,    1, 3512,    2, 0x06,  370 /* Public */,
     172,    2, 3515,    2, 0x06,  372 /* Public */,
     173,    0, 3520,    2, 0x06,  375 /* Public */,
     174,    2, 3521,    2, 0x06,  376 /* Public */,
     175,    1, 3526,    2, 0x06,  379 /* Public */,
     176,    2, 3529,    2, 0x06,  381 /* Public */,
     177,    0, 3534,    2, 0x06,  384 /* Public */,
     178,    2, 3535,    2, 0x06,  385 /* Public */,
     179,    1, 3540,    2, 0x06,  388 /* Public */,
     180,    2, 3543,    2, 0x06,  390 /* Public */,
     181,    0, 3548,    2, 0x06,  393 /* Public */,
     182,    2, 3549,    2, 0x06,  394 /* Public */,
     183,    1, 3554,    2, 0x06,  397 /* Public */,
     184,    2, 3557,    2, 0x06,  399 /* Public */,
     185,    0, 3562,    2, 0x06,  402 /* Public */,
     186,    2, 3563,    2, 0x06,  403 /* Public */,
     187,    0, 3568,    2, 0x06,  406 /* Public */,
     188,    2, 3569,    2, 0x06,  407 /* Public */,
     189,    1, 3574,    2, 0x06,  410 /* Public */,
     190,    2, 3577,    2, 0x06,  412 /* Public */,
     191,    1, 3582,    2, 0x06,  415 /* Public */,
     192,    2, 3585,    2, 0x06,  417 /* Public */,
     193,    1, 3590,    2, 0x06,  420 /* Public */,
     194,    2, 3593,    2, 0x06,  422 /* Public */,
     195,    0, 3598,    2, 0x06,  425 /* Public */,
     196,    2, 3599,    2, 0x06,  426 /* Public */,
     197,    1, 3604,    2, 0x06,  429 /* Public */,
     198,    2, 3607,    2, 0x06,  431 /* Public */,
     199,    1, 3612,    2, 0x06,  434 /* Public */,
     200,    2, 3615,    2, 0x06,  436 /* Public */,
     201,    1, 3620,    2, 0x06,  439 /* Public */,
     202,    2, 3623,    2, 0x06,  441 /* Public */,
     203,    1, 3628,    2, 0x06,  444 /* Public */,
     204,    2, 3631,    2, 0x06,  446 /* Public */,
     205,    1, 3636,    2, 0x06,  449 /* Public */,
     206,    2, 3639,    2, 0x06,  451 /* Public */,
     207,    1, 3644,    2, 0x06,  454 /* Public */,
     208,    2, 3647,    2, 0x06,  456 /* Public */,
     209,    0, 3652,    2, 0x06,  459 /* Public */,
     210,    2, 3653,    2, 0x06,  460 /* Public */,
     211,    1, 3658,    2, 0x06,  463 /* Public */,
     212,    2, 3661,    2, 0x06,  465 /* Public */,
     213,    1, 3666,    2, 0x06,  468 /* Public */,
     214,    2, 3669,    2, 0x06,  470 /* Public */,
     215,    0, 3674,    2, 0x06,  473 /* Public */,
     216,    2, 3675,    2, 0x06,  474 /* Public */,
     217,    1, 3680,    2, 0x06,  477 /* Public */,
     218,    2, 3683,    2, 0x06,  479 /* Public */,
     219,    1, 3688,    2, 0x06,  482 /* Public */,
     220,    2, 3691,    2, 0x06,  484 /* Public */,
     221,    0, 3696,    2, 0x06,  487 /* Public */,
     222,    2, 3697,    2, 0x06,  488 /* Public */,
     223,    1, 3702,    2, 0x06,  491 /* Public */,
     224,    2, 3705,    2, 0x06,  493 /* Public */,
     225,    1, 3710,    2, 0x06,  496 /* Public */,
     226,    2, 3713,    2, 0x06,  498 /* Public */,
     227,    1, 3718,    2, 0x06,  501 /* Public */,
     228,    2, 3721,    2, 0x06,  503 /* Public */,
     229,    0, 3726,    2, 0x06,  506 /* Public */,
     230,    2, 3727,    2, 0x06,  507 /* Public */,
     231,    1, 3732,    2, 0x06,  510 /* Public */,
     232,    2, 3735,    2, 0x06,  512 /* Public */,
     233,    1, 3740,    2, 0x06,  515 /* Public */,
     234,    2, 3743,    2, 0x06,  517 /* Public */,
     235,    1, 3748,    2, 0x06,  520 /* Public */,
     236,    2, 3751,    2, 0x06,  522 /* Public */,
     237,    1, 3756,    2, 0x06,  525 /* Public */,
     238,    2, 3759,    2, 0x06,  527 /* Public */,
     239,    0, 3764,    2, 0x06,  530 /* Public */,
     240,    2, 3765,    2, 0x06,  531 /* Public */,
     241,    1, 3770,    2, 0x06,  534 /* Public */,
     242,    2, 3773,    2, 0x06,  536 /* Public */,
     243,    0, 3778,    2, 0x06,  539 /* Public */,
     244,    2, 3779,    2, 0x06,  540 /* Public */,
     245,    0, 3784,    2, 0x06,  543 /* Public */,
     246,    2, 3785,    2, 0x06,  544 /* Public */,
     247,    1, 3790,    2, 0x06,  547 /* Public */,
     248,    2, 3793,    2, 0x06,  549 /* Public */,
     249,    1, 3798,    2, 0x06,  552 /* Public */,
     250,    2, 3801,    2, 0x06,  554 /* Public */,
     251,    1, 3806,    2, 0x06,  557 /* Public */,
     252,    2, 3809,    2, 0x06,  559 /* Public */,
     253,    1, 3814,    2, 0x06,  562 /* Public */,
     254,    2, 3817,    2, 0x06,  564 /* Public */,
     255,    1, 3822,    2, 0x06,  567 /* Public */,
     256,    2, 3825,    2, 0x06,  569 /* Public */,
     257,    1, 3830,    2, 0x06,  572 /* Public */,
     258,    2, 3833,    2, 0x06,  574 /* Public */,
     259,    1, 3838,    2, 0x06,  577 /* Public */,
     260,    2, 3841,    2, 0x06,  579 /* Public */,
     261,    0, 3846,    2, 0x06,  582 /* Public */,
     262,    2, 3847,    2, 0x06,  583 /* Public */,
     263,    1, 3852,    2, 0x06,  586 /* Public */,
     264,    2, 3855,    2, 0x06,  588 /* Public */,
     265,    0, 3860,    2, 0x06,  591 /* Public */,
     266,    2, 3861,    2, 0x06,  592 /* Public */,
     267,    0, 3866,    2, 0x06,  595 /* Public */,
     268,    2, 3867,    2, 0x06,  596 /* Public */,
     269,    1, 3872,    2, 0x06,  599 /* Public */,
     270,    2, 3875,    2, 0x06,  601 /* Public */,
     271,    1, 3880,    2, 0x06,  604 /* Public */,
     272,    2, 3883,    2, 0x06,  606 /* Public */,
     273,    1, 3888,    2, 0x06,  609 /* Public */,
     274,    2, 3891,    2, 0x06,  611 /* Public */,
     275,    0, 3896,    2, 0x06,  614 /* Public */,
     276,    2, 3897,    2, 0x06,  615 /* Public */,
     277,    0, 3902,    2, 0x06,  618 /* Public */,
     278,    2, 3903,    2, 0x06,  619 /* Public */,
     279,    0, 3908,    2, 0x06,  622 /* Public */,
     280,    2, 3909,    2, 0x06,  623 /* Public */,
     281,    0, 3914,    2, 0x06,  626 /* Public */,
     282,    2, 3915,    2, 0x06,  627 /* Public */,
     283,    1, 3920,    2, 0x06,  630 /* Public */,
     284,    2, 3923,    2, 0x06,  632 /* Public */,
     285,    1, 3928,    2, 0x06,  635 /* Public */,
     286,    2, 3931,    2, 0x06,  637 /* Public */,
     287,    0, 3936,    2, 0x06,  640 /* Public */,
     288,    2, 3937,    2, 0x06,  641 /* Public */,
     289,    1, 3942,    2, 0x06,  644 /* Public */,
     290,    2, 3945,    2, 0x06,  646 /* Public */,
     291,    1, 3950,    2, 0x06,  649 /* Public */,
     292,    2, 3953,    2, 0x06,  651 /* Public */,
     293,    0, 3958,    2, 0x06,  654 /* Public */,
     294,    2, 3959,    2, 0x06,  655 /* Public */,
     295,    1, 3964,    2, 0x06,  658 /* Public */,
     296,    2, 3967,    2, 0x06,  660 /* Public */,
     297,    1, 3972,    2, 0x06,  663 /* Public */,
     298,    2, 3975,    2, 0x06,  665 /* Public */,
     299,    0, 3980,    2, 0x06,  668 /* Public */,
     300,    2, 3981,    2, 0x06,  669 /* Public */,
     301,    0, 3986,    2, 0x06,  672 /* Public */,
     302,    2, 3987,    2, 0x06,  673 /* Public */,
     303,    0, 3992,    2, 0x06,  676 /* Public */,
     304,    2, 3993,    2, 0x06,  677 /* Public */,
     305,    1, 3998,    2, 0x06,  680 /* Public */,
     306,    2, 4001,    2, 0x06,  682 /* Public */,
     307,    1, 4006,    2, 0x06,  685 /* Public */,
     308,    2, 4009,    2, 0x06,  687 /* Public */,
     309,    1, 4014,    2, 0x06,  690 /* Public */,
     310,    2, 4017,    2, 0x06,  692 /* Public */,
     311,    1, 4022,    2, 0x06,  695 /* Public */,
     312,    2, 4025,    2, 0x06,  697 /* Public */,
     313,    0, 4030,    2, 0x06,  700 /* Public */,
     314,    2, 4031,    2, 0x06,  701 /* Public */,
     315,    1, 4036,    2, 0x06,  704 /* Public */,
     316,    2, 4039,    2, 0x06,  706 /* Public */,
     317,    0, 4044,    2, 0x06,  709 /* Public */,
     318,    2, 4045,    2, 0x06,  710 /* Public */,
     319,    1, 4050,    2, 0x06,  713 /* Public */,
     320,    2, 4053,    2, 0x06,  715 /* Public */,
     321,    0, 4058,    2, 0x06,  718 /* Public */,
     322,    2, 4059,    2, 0x06,  719 /* Public */,
     323,    1, 4064,    2, 0x06,  722 /* Public */,
     324,    2, 4067,    2, 0x06,  724 /* Public */,
     325,    1, 4072,    2, 0x06,  727 /* Public */,
     326,    2, 4075,    2, 0x06,  729 /* Public */,
     327,    0, 4080,    2, 0x06,  732 /* Public */,
     328,    2, 4081,    2, 0x06,  733 /* Public */,
     329,    1, 4086,    2, 0x06,  736 /* Public */,
     330,    2, 4089,    2, 0x06,  738 /* Public */,
     331,    1, 4094,    2, 0x06,  741 /* Public */,
     332,    2, 4097,    2, 0x06,  743 /* Public */,
     333,    1, 4102,    2, 0x06,  746 /* Public */,
     334,    2, 4105,    2, 0x06,  748 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
     335,    3, 4110,    2, 0x09,  751 /* Protected */,
     338,    3, 4117,    2, 0x09,  755 /* Protected */,
     339,    1, 4124,    2, 0x0a,  759 /* Public */,
     342,    2, 4127,    2, 0x0a,  761 /* Public */,
     344,    3, 4132,    2, 0x0a,  764 /* Public */,
     346,    2, 4139,    2, 0x0a,  768 /* Public */,
     347,    5, 4144,    2, 0x0a,  771 /* Public */,
     352,    4, 4155,    2, 0x0a,  777 /* Public */,
     354,    6, 4164,    2, 0x0a,  782 /* Public */,
     356,    5, 4177,    2, 0x0a,  789 /* Public */,
     357,    2, 4188,    2, 0x0a,  795 /* Public */,
     358,    1, 4193,    2, 0x0a,  798 /* Public */,
     359,    3, 4196,    2, 0x0a,  800 /* Public */,
     361,    2, 4203,    2, 0x0a,  804 /* Public */,
     362,    3, 4208,    2, 0x0a,  807 /* Public */,
     363,    2, 4215,    2, 0x0a,  811 /* Public */,
     364,    2, 4220,    2, 0x0a,  814 /* Public */,
     365,    3, 4225,    2, 0x0a,  817 /* Public */,
     366,    1, 4232,    2, 0x0a,  821 /* Public */,
     367,    1, 4235,    2, 0x0a,  823 /* Public */,
     368,    2, 4238,    2, 0x0a,  825 /* Public */,
     369,    1, 4243,    2, 0x0a,  828 /* Public */,
     370,    3, 4246,    2, 0x0a,  830 /* Public */,
     373,    2, 4253,    2, 0x0a,  834 /* Public */,
     374,    6, 4258,    2, 0x0a,  837 /* Public */,
     379,    4, 4271,    2, 0x0a,  844 /* Public */,
     382,    1, 4280,    2, 0x0a,  849 /* Public */,
     384,    2, 4283,    2, 0x0a,  851 /* Public */,
     385,    3, 4288,    2, 0x0a,  854 /* Public */,
     386,    2, 4295,    2, 0x0a,  858 /* Public */,
     387,    3, 4300,    2, 0x0a,  861 /* Public */,
     388,    2, 4307,    2, 0x0a,  865 /* Public */,
     389,    1, 4312,    2, 0x0a,  868 /* Public */,
     390,    1, 4315,    2, 0x0a,  870 /* Public */,
     391,    1, 4318,    2, 0x0a,  872 /* Public */,
     392,    3, 4321,    2, 0x0a,  874 /* Public */,
     396,    3, 4328,    2, 0x0a,  878 /* Public */,
     398,    0, 4335,    2, 0x0a,  882 /* Public */,
     399,    2, 4336,    2, 0x0a,  883 /* Public */,
     400,    2, 4341,    2, 0x0a,  886 /* Public */,
     401,    1, 4346,    2, 0x0a,  889 /* Public */,
     403,    3, 4349,    2, 0x0a,  891 /* Public */,
     407,    1, 4356,    2, 0x0a,  895 /* Public */,
     408,    4, 4359,    2, 0x0a,  897 /* Public */,
     413,    1, 4368,    2, 0x0a,  902 /* Public */,
     414,    1, 4371,    2, 0x0a,  904 /* Public */,
     416,    2, 4374,    2, 0x0a,  906 /* Public */,
     418,    1, 4379,    2, 0x0a,  909 /* Public */,
     419,    0, 4382,    2, 0x0a,  911 /* Public */,
     420,    1, 4383,    2, 0x0a,  912 /* Public */,
     422,    0, 4386,    2, 0x0a,  914 /* Public */,
     423,    1, 4387,    2, 0x0a,  915 /* Public */,
     424,    0, 4390,    2, 0x0a,  917 /* Public */,
     425,    1, 4391,    2, 0x0a,  918 /* Public */,
     426,    2, 4394,    2, 0x0a,  920 /* Public */,
     427,    1, 4399,    2, 0x0a,  923 /* Public */,
     428,    2, 4402,    2, 0x0a,  925 /* Public */,
     429,    1, 4407,    2, 0x0a,  928 /* Public */,
     430,    1, 4410,    2, 0x0a,  930 /* Public */,
     431,    1, 4413,    2, 0x0a,  932 /* Public */,
     432,    2, 4416,    2, 0x0a,  934 /* Public */,
     433,    1, 4421,    2, 0x0a,  937 /* Public */,
     434,    1, 4424,    2, 0x0a,  939 /* Public */,
     435,    1, 4427,    2, 0x0a,  941 /* Public */,
     436,    0, 4430,    2, 0x0a,  943 /* Public */,
     437,    0, 4431,    2, 0x0a,  944 /* Public */,
     438,    1, 4432,    2, 0x0a,  945 /* Public */,
     439,    0, 4435,    2, 0x0a,  947 /* Public */,
     440,    0, 4436,    2, 0x0a,  948 /* Public */,
     441,    1, 4437,    2, 0x0a,  949 /* Public */,
     443,    0, 4440,    2, 0x0a,  951 /* Public */,
     444,    0, 4441,    2, 0x0a,  952 /* Public */,
     445,    1, 4442,    2, 0x0a,  953 /* Public */,
     446,    1, 4445,    2, 0x0a,  955 /* Public */,
     448,    1, 4448,    2, 0x0a,  957 /* Public */,
     449,    1, 4451,    2, 0x0a,  959 /* Public */,
     450,    0, 4454,    2, 0x0a,  961 /* Public */,
     451,    0, 4455,    2, 0x0a,  962 /* Public */,
     452,    0, 4456,    2, 0x0a,  963 /* Public */,
     453,    1, 4457,    2, 0x0a,  964 /* Public */,
     455,    1, 4460,    2, 0x0a,  966 /* Public */,
     456,    1, 4463,    2, 0x0a,  968 /* Public */,
     457,    2, 4466,    2, 0x0a,  970 /* Public */,
     459,    1, 4471,    2, 0x0a,  973 /* Public */,
     460,    2, 4474,    2, 0x0a,  975 /* Public */,
     461,    1, 4479,    2, 0x0a,  978 /* Public */,
     462,    2, 4482,    2, 0x0a,  980 /* Public */,
     463,    2, 4487,    2, 0x0a,  983 /* Public */,
     464,    3, 4492,    2, 0x0a,  986 /* Public */,
     465,    2, 4499,    2, 0x0a,  990 /* Public */,
     466,    3, 4504,    2, 0x0a,  993 /* Public */,
     467,    2, 4511,    2, 0x0a,  997 /* Public */,
     468,    2, 4516,    2, 0x0a, 1000 /* Public */,
     469,    3, 4521,    2, 0x0a, 1003 /* Public */,
     470,    1, 4528,    2, 0x0a, 1007 /* Public */,
     471,    1, 4531,    2, 0x0a, 1009 /* Public */,
     472,    1, 4534,    2, 0x0a, 1011 /* Public */,
     473,    1, 4537,    2, 0x0a, 1013 /* Public */,
     474,    1, 4540,    2, 0x0a, 1015 /* Public */,
     475,    4, 4543,    2, 0x0a, 1017 /* Public */,
     476,    5, 4552,    2, 0x0a, 1022 /* Public */,
     477,    2, 4563,    2, 0x0a, 1028 /* Public */,
     478,    2, 4568,    2, 0x0a, 1031 /* Public */,
     479,    3, 4573,    2, 0x0a, 1034 /* Public */,
     480,    1, 4580,    2, 0x0a, 1038 /* Public */,
     481,    1, 4583,    2, 0x0a, 1040 /* Public */,
     482,    2, 4586,    2, 0x0a, 1042 /* Public */,
     483,    1, 4591,    2, 0x0a, 1045 /* Public */,
     484,    2, 4594,    2, 0x0a, 1047 /* Public */,
     485,    6, 4599,    2, 0x0a, 1050 /* Public */,
     486,    4, 4612,    2, 0x0a, 1057 /* Public */,
     487,    1, 4621,    2, 0x0a, 1062 /* Public */,
     488,    2, 4624,    2, 0x0a, 1064 /* Public */,
     489,    1, 4629,    2, 0x0a, 1067 /* Public */,
     490,    1, 4632,    2, 0x0a, 1069 /* Public */,
     491,    0, 4635,    2, 0x0a, 1071 /* Public */,
     492,    2, 4636,    2, 0x0a, 1072 /* Public */,
     493,    1, 4641,    2, 0x0a, 1075 /* Public */,
     494,    3, 4644,    2, 0x0a, 1077 /* Public */,
     495,    1, 4651,    2, 0x0a, 1081 /* Public */,
     496,    4, 4654,    2, 0x0a, 1083 /* Public */,
     497,    1, 4663,    2, 0x0a, 1088 /* Public */,
     498,    1, 4666,    2, 0x0a, 1090 /* Public */,
     499,    2, 4669,    2, 0x0a, 1092 /* Public */,
     500,    1, 4674,    2, 0x0a, 1095 /* Public */,
     501,    0, 4677,    2, 0x0a, 1097 /* Public */,
     502,    1, 4678,    2, 0x0a, 1098 /* Public */,
     503,    0, 4681,    2, 0x0a, 1100 /* Public */,
     504,    1, 4682,    2, 0x0a, 1101 /* Public */,
     505,    0, 4685,    2, 0x0a, 1103 /* Public */,
     506,    1, 4686,    2, 0x0a, 1104 /* Public */,
     507,    1, 4689,    2, 0x0a, 1106 /* Public */,
     508,    1, 4692,    2, 0x0a, 1108 /* Public */,
     509,    2, 4695,    2, 0x0a, 1110 /* Public */,
     510,    1, 4700,    2, 0x0a, 1113 /* Public */,
     511,    1, 4703,    2, 0x0a, 1115 /* Public */,
     512,    1, 4706,    2, 0x0a, 1117 /* Public */,
     513,    0, 4709,    2, 0x0a, 1119 /* Public */,
     514,    0, 4710,    2, 0x0a, 1120 /* Public */,
     515,    1, 4711,    2, 0x0a, 1121 /* Public */,
     516,    0, 4714,    2, 0x0a, 1123 /* Public */,
     517,    0, 4715,    2, 0x0a, 1124 /* Public */,
     518,    1, 4716,    2, 0x0a, 1125 /* Public */,
     519,    0, 4719,    2, 0x0a, 1127 /* Public */,
     520,    0, 4720,    2, 0x0a, 1128 /* Public */,
     521,    1, 4721,    2, 0x0a, 1129 /* Public */,
     522,    1, 4724,    2, 0x0a, 1131 /* Public */,
     523,    1, 4727,    2, 0x0a, 1133 /* Public */,
     524,    1, 4730,    2, 0x0a, 1135 /* Public */,
     525,    0, 4733,    2, 0x0a, 1137 /* Public */,
     526,    1, 4734,    2, 0x0a, 1138 /* Public */,
     527,    1, 4737,    2, 0x0a, 1140 /* Public */,
     528,    2, 4740,    2, 0x0a, 1142 /* Public */,
     529,    1, 4745,    2, 0x0a, 1145 /* Public */,
     530,    2, 4748,    2, 0x0a, 1147 /* Public */,
     531,    1, 4753,    2, 0x0a, 1150 /* Public */,
     532,    2, 4756,    2, 0x0a, 1152 /* Public */,
     533,    2, 4761,    2, 0x0a, 1155 /* Public */,
     534,    2, 4766,    2, 0x0a, 1158 /* Public */,
     535,    3, 4771,    2, 0x0a, 1161 /* Public */,
     536,    1, 4778,    2, 0x0a, 1165 /* Public */,
     537,    1, 4781,    2, 0x0a, 1167 /* Public */,
     538,    1, 4784,    2, 0x0a, 1169 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    3,    5,
    QMetaType::Void, QMetaType::QByteArray, 0x80000000 | 8,    7,    9,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArrayList,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArrayList,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 60,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArrayList,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 60,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 60,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 60,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 161,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 161,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArrayList,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 60,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArrayList,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 60,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 60,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 60,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 161,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    2,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::UInt, QMetaType::ULongLong, 0x80000000 | 4,  336,  337,    2,
    QMetaType::Void, QMetaType::UInt, QMetaType::ULongLong, 0x80000000 | 4,  336,  337,    2,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11,  341,  343,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, QMetaType::QByteArray,  341,  343,  345,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11,  341,  343,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool, QMetaType::Bool,  341,  348,  349,  350,  351,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool,  341,  348,  349,  353,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool, QMetaType::Bool, QMetaType::QByteArrayList,  341,  348,  349,  350,  351,  355,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool, QMetaType::QByteArrayList,  341,  348,  349,  353,  355,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  341,    7,  360,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  341,    7,  360,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  341,    7,  360,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, QMetaType::QByteArrayList,  341,  371,  372,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11,  341,  375,  376,  345,  377,  378,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11,  341,  375,  380,  381,
    0x80000000 | 340, 0x80000000 | 11,  383,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  383,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  383,    7,  360,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  383,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  383,    7,  360,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  383,    7,
    0x80000000 | 340, 0x80000000 | 11,  383,
    0x80000000 | 340, 0x80000000 | 11,  383,
    0x80000000 | 340, 0x80000000 | 11,  383,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 161,  393,  394,  395,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool,  393,  394,  397,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11,  393,  394,
    0x80000000 | 340, QMetaType::QByteArray, 0x80000000 | 4,    7,  360,
    0x80000000 | 340, QMetaType::QByteArray,  402,
    0x80000000 | 340, QMetaType::QByteArray, QMetaType::QByteArray, QMetaType::Bool,  404,  405,  406,
    0x80000000 | 340, QMetaType::QByteArray,  404,
    0x80000000 | 340, QMetaType::QByteArray, QMetaType::Bool, QMetaType::Bool, QMetaType::Bool,  409,  410,  411,  412,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340, QMetaType::QByteArray,  415,
    0x80000000 | 340, QMetaType::QByteArray, 0x80000000 | 8,  417,    9,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340,
    0x80000000 | 340, QMetaType::QByteArray,  421,
    0x80000000 | 340,
    0x80000000 | 340, QMetaType::QByteArray,  345,
    0x80000000 | 340,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340, QMetaType::QByteArray, 0x80000000 | 4,    7,  360,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340, QMetaType::QByteArray, 0x80000000 | 4,    7,  360,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340, QMetaType::QByteArray, 0x80000000 | 4,    7,  360,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 11,  383,
    0x80000000 | 340, QMetaType::QByteArray,  447,
    0x80000000 | 340, QMetaType::QByteArray,  447,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340,
    0x80000000 | 340,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 8,  454,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QPoint,  442,  458,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11,  442,  394,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11,  442,  393,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  442,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  442,    7,  360,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  442,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  442,    7,  360,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  442,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  442,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  442,    7,  360,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool,  341,  348,  349,  353,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool, QMetaType::QByteArrayList,  341,  348,  349,  353,  355,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  341,    7,  360,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  341,    7,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11,  341,  375,  376,  345,  377,  378,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11,  341,  375,  380,  381,
    0x80000000 | 340, 0x80000000 | 11,  383,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  383,    7,
    0x80000000 | 340, 0x80000000 | 11,  383,
    0x80000000 | 340, 0x80000000 | 11,  383,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11,  393,  394,
    0x80000000 | 340, QMetaType::QByteArray,  402,
    0x80000000 | 340, QMetaType::QByteArray, QMetaType::QByteArray, QMetaType::Bool,  404,  405,  406,
    0x80000000 | 340, QMetaType::QByteArray,  404,
    0x80000000 | 340, QMetaType::QByteArray, QMetaType::Bool, QMetaType::Bool, QMetaType::Bool,  409,  410,  411,  412,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340, QMetaType::QByteArray,  415,
    0x80000000 | 340, QMetaType::QByteArray, 0x80000000 | 8,  417,    9,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340,
    0x80000000 | 340, QMetaType::QByteArray,  421,
    0x80000000 | 340,
    0x80000000 | 340, QMetaType::QByteArray,  345,
    0x80000000 | 340,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340, QMetaType::QByteArray, 0x80000000 | 4,    7,  360,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340, QMetaType::QByteArray,  409,
    0x80000000 | 340,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 11,  341,
    0x80000000 | 340,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 11,  383,
    0x80000000 | 340, QMetaType::QByteArray,  447,
    0x80000000 | 340, QMetaType::QByteArray,  447,
    0x80000000 | 340, QMetaType::QByteArray,    7,
    0x80000000 | 340,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QPoint,  442,  458,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11,  442,  394,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11, 0x80000000 | 11,  442,  393,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  442,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray,  442,    7,
    0x80000000 | 340, 0x80000000 | 11, QMetaType::QByteArray, 0x80000000 | 4,  442,    7,  360,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11,  442,
    0x80000000 | 340, 0x80000000 | 11,  442,

 // enums: name, alias, flags, count, data
     539,  539, 0x0,  162, 4792,

 // enum data: key, value
     540, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NULL),
     541, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_LINE_COUNT),
     542, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_GET_LINE),
     543, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_SET_LINE),
     544, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_DEL_LINE),
     545, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_GET_LINE_SLICE),
     546, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_GET_LINES),
     547, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_SET_LINE_SLICE),
     548, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_SET_LINES),
     549, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_GET_VAR),
     550, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_GET_CHANGEDTICK),
     551, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_SET_VAR),
     552, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_DEL_VAR),
     553, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_SET_VAR),
     554, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_DEL_VAR),
     555, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_GET_OPTION),
     556, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_SET_OPTION),
     557, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_GET_NUMBER),
     558, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_GET_NAME),
     559, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_SET_NAME),
     560, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_IS_VALID),
     561, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_INSERT),
     562, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_GET_MARK),
     563, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_ADD_HIGHLIGHT),
     564, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_BUF_CLEAR_HIGHLIGHT),
     565, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_TABPAGE_LIST_WINS),
     566, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_TABPAGE_GET_VAR),
     567, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_TABPAGE_SET_VAR),
     568, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_TABPAGE_DEL_VAR),
     569, uint(NeovimQt::NeovimApi2::NEOVIM_FN_TABPAGE_SET_VAR),
     570, uint(NeovimQt::NeovimApi2::NEOVIM_FN_TABPAGE_DEL_VAR),
     571, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_TABPAGE_GET_WIN),
     572, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_TABPAGE_GET_NUMBER),
     573, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_TABPAGE_IS_VALID),
     574, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_UI_ATTACH),
     575, uint(NeovimQt::NeovimApi2::NEOVIM_FN_UI_ATTACH),
     576, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_UI_DETACH),
     577, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_UI_TRY_RESIZE),
     578, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_UI_SET_OPTION),
     579, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_COMMAND),
     580, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_FEEDKEYS),
     581, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_INPUT),
     582, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_REPLACE_TERMCODES),
     583, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_COMMAND_OUTPUT),
     584, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_EVAL),
     585, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_CALL_FUNCTION),
     586, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_STRWIDTH),
     587, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_LIST_RUNTIME_PATHS),
     588, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_SET_CURRENT_DIR),
     589, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_CURRENT_LINE),
     590, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_SET_CURRENT_LINE),
     591, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_DEL_CURRENT_LINE),
     592, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_VAR),
     593, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_SET_VAR),
     594, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_DEL_VAR),
     595, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_SET_VAR),
     596, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_DEL_VAR),
     597, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_VVAR),
     598, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_OPTION),
     599, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_SET_OPTION),
     600, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_OUT_WRITE),
     601, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_ERR_WRITE),
     602, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_ERR_WRITELN),
     603, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_LIST_BUFS),
     604, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_CURRENT_BUF),
     605, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_SET_CURRENT_BUF),
     606, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_LIST_WINS),
     607, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_CURRENT_WIN),
     608, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_SET_CURRENT_WIN),
     609, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_LIST_TABPAGES),
     610, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_CURRENT_TABPAGE),
     611, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_SET_CURRENT_TABPAGE),
     612, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_SUBSCRIBE),
     613, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_UNSUBSCRIBE),
     614, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_COLOR_BY_NAME),
     615, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_COLOR_MAP),
     616, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_MODE),
     617, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_GET_API_INFO),
     618, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_CALL_ATOMIC),
     619, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_GET_BUF),
     620, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_GET_CURSOR),
     621, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_SET_CURSOR),
     622, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_GET_HEIGHT),
     623, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_SET_HEIGHT),
     624, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_GET_WIDTH),
     625, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_SET_WIDTH),
     626, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_GET_VAR),
     627, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_SET_VAR),
     628, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_DEL_VAR),
     629, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_SET_VAR),
     630, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_DEL_VAR),
     631, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_GET_OPTION),
     632, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_SET_OPTION),
     633, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_GET_POSITION),
     634, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_GET_TABPAGE),
     635, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_GET_NUMBER),
     636, uint(NeovimQt::NeovimApi2::NEOVIM_FN_NVIM_WIN_IS_VALID),
     637, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_LINE_COUNT),
     638, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_GET_LINES),
     639, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_SET_LINES),
     640, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_GET_VAR),
     641, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_GET_OPTION),
     642, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_SET_OPTION),
     643, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_GET_NUMBER),
     644, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_GET_NAME),
     645, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_SET_NAME),
     646, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_IS_VALID),
     647, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_GET_MARK),
     648, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_ADD_HIGHLIGHT),
     649, uint(NeovimQt::NeovimApi2::NEOVIM_FN_BUFFER_CLEAR_HIGHLIGHT),
     650, uint(NeovimQt::NeovimApi2::NEOVIM_FN_TABPAGE_GET_WINDOWS),
     651, uint(NeovimQt::NeovimApi2::NEOVIM_FN_TABPAGE_GET_VAR),
     652, uint(NeovimQt::NeovimApi2::NEOVIM_FN_TABPAGE_GET_WINDOW),
     653, uint(NeovimQt::NeovimApi2::NEOVIM_FN_TABPAGE_IS_VALID),
     654, uint(NeovimQt::NeovimApi2::NEOVIM_FN_UI_DETACH),
     655, uint(NeovimQt::NeovimApi2::NEOVIM_FN_UI_TRY_RESIZE),
     656, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_COMMAND),
     657, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_FEEDKEYS),
     658, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_INPUT),
     659, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_REPLACE_TERMCODES),
     660, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_COMMAND_OUTPUT),
     661, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_EVAL),
     662, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_CALL_FUNCTION),
     663, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_STRWIDTH),
     664, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_LIST_RUNTIME_PATHS),
     665, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_CHANGE_DIRECTORY),
     666, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_CURRENT_LINE),
     667, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_SET_CURRENT_LINE),
     668, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_DEL_CURRENT_LINE),
     669, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_VAR),
     670, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_VVAR),
     671, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_OPTION),
     672, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_SET_OPTION),
     673, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_OUT_WRITE),
     674, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_ERR_WRITE),
     675, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_REPORT_ERROR),
     676, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_BUFFERS),
     677, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_CURRENT_BUFFER),
     678, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_SET_CURRENT_BUFFER),
     679, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_WINDOWS),
     680, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_CURRENT_WINDOW),
     681, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_SET_CURRENT_WINDOW),
     682, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_TABPAGES),
     683, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_CURRENT_TABPAGE),
     684, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_SET_CURRENT_TABPAGE),
     685, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_SUBSCRIBE),
     686, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_UNSUBSCRIBE),
     687, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_NAME_TO_COLOR),
     688, uint(NeovimQt::NeovimApi2::NEOVIM_FN_VIM_GET_COLOR_MAP),
     689, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_GET_BUFFER),
     690, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_GET_CURSOR),
     691, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_SET_CURSOR),
     692, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_GET_HEIGHT),
     693, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_SET_HEIGHT),
     694, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_GET_WIDTH),
     695, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_SET_WIDTH),
     696, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_GET_VAR),
     697, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_GET_OPTION),
     698, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_SET_OPTION),
     699, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_GET_POSITION),
     700, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_GET_TABPAGE),
     701, uint(NeovimQt::NeovimApi2::NEOVIM_FN_WINDOW_IS_VALID),

       0        // eod
};

Q_CONSTINIT const QMetaObject NeovimQt::NeovimApi2::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ZN8NeovimQt10NeovimApi2E.offsetsAndSizes,
    qt_meta_data_ZN8NeovimQt10NeovimApi2E,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN8NeovimQt10NeovimApi2E_t,
        // enum 'FunctionId'
        QtPrivate::TypeAndForceComplete<NeovimApi2::FunctionId, std::true_type>,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<NeovimApi2, std::true_type>,
        // method 'error'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'neovimNotification'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QByteArray &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariantList &, std::false_type>,
        // method 'on_nvim_buf_line_count'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_buf_line_count'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_get_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'err_buffer_get_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_set_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_buffer_set_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_del_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_buffer_del_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_get_line_slice'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<QByteArray>, std::false_type>,
        // method 'err_buffer_get_line_slice'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_get_lines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<QByteArray>, std::false_type>,
        // method 'err_nvim_buf_get_lines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_set_line_slice'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_buffer_set_line_slice'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_set_lines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_buf_set_lines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_buf_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_get_changedtick'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_buf_get_changedtick'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_buf_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_buf_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_buffer_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_buffer_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_buf_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_buf_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_get_number'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_buf_get_number'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_get_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'err_nvim_buf_get_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_set_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_buf_set_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'err_nvim_buf_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_insert'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_buffer_insert'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_get_mark'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'err_nvim_buf_get_mark'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_add_highlight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_buf_add_highlight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_buf_clear_highlight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_buf_clear_highlight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_tabpage_list_wins'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<int64_t>, std::false_type>,
        // method 'err_nvim_tabpage_list_wins'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_tabpage_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_tabpage_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_tabpage_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_tabpage_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_tabpage_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_tabpage_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_tabpage_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_tabpage_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_tabpage_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_tabpage_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_tabpage_get_win'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_tabpage_get_win'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_tabpage_get_number'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_tabpage_get_number'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_tabpage_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'err_nvim_tabpage_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_ui_attach'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_ui_attach'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_ui_attach'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_ui_attach'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_ui_detach'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_ui_detach'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_ui_try_resize'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_ui_try_resize'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_ui_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_ui_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_command'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_command'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_feedkeys'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_feedkeys'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_input'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_input'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_replace_termcodes'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'err_nvim_replace_termcodes'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_command_output'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'err_nvim_command_output'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_eval'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_eval'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_call_function'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_call_function'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_strwidth'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_strwidth'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_list_runtime_paths'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<QByteArray>, std::false_type>,
        // method 'err_nvim_list_runtime_paths'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_set_current_dir'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_set_current_dir'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'err_nvim_get_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_set_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_set_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_del_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_del_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_vim_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_vim_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_vvar'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_get_vvar'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_out_write'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_out_write'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_err_write'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_err_write'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_err_writeln'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_err_writeln'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_list_bufs'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<int64_t>, std::false_type>,
        // method 'err_nvim_list_bufs'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_current_buf'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_get_current_buf'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_set_current_buf'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_set_current_buf'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_list_wins'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<int64_t>, std::false_type>,
        // method 'err_nvim_list_wins'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_current_win'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_get_current_win'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_set_current_win'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_set_current_win'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_list_tabpages'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<int64_t>, std::false_type>,
        // method 'err_nvim_list_tabpages'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_current_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_get_current_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_set_current_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_set_current_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_subscribe'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_subscribe'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_unsubscribe'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_unsubscribe'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_color_by_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_get_color_by_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_color_map'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariantMap, std::false_type>,
        // method 'err_nvim_get_color_map'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_mode'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariantMap, std::false_type>,
        // method 'err_nvim_get_mode'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_get_api_info'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariantList, std::false_type>,
        // method 'err_nvim_get_api_info'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_call_atomic'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariantList, std::false_type>,
        // method 'err_nvim_call_atomic'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_get_buf'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_win_get_buf'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_get_cursor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'err_nvim_win_get_cursor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_set_cursor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_win_set_cursor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_get_height'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_win_get_height'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_set_height'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_win_set_height'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_get_width'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_win_get_width'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_set_width'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_win_set_width'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_win_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_win_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_win_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_window_set_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_window_del_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_nvim_win_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_nvim_win_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_get_position'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'err_nvim_win_get_position'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_get_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_win_get_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_get_number'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_nvim_win_get_number'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_nvim_win_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'err_nvim_win_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_line_count'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_buffer_line_count'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_get_lines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<QByteArray>, std::false_type>,
        // method 'err_buffer_get_lines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_set_lines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_buffer_set_lines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_buffer_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_buffer_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_buffer_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_get_number'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_buffer_get_number'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_get_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'err_buffer_get_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_set_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_buffer_set_name'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'err_buffer_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_get_mark'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'err_buffer_get_mark'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_add_highlight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_buffer_add_highlight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_buffer_clear_highlight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_buffer_clear_highlight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_tabpage_get_windows'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<int64_t>, std::false_type>,
        // method 'err_tabpage_get_windows'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_tabpage_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_tabpage_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_tabpage_get_window'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_tabpage_get_window'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_tabpage_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'err_tabpage_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_ui_detach'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_ui_detach'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_ui_try_resize'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_ui_try_resize'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_command'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_command'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_feedkeys'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_feedkeys'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_input'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_vim_input'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_replace_termcodes'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'err_vim_replace_termcodes'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_command_output'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'err_vim_command_output'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_eval'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_vim_eval'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_call_function'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_vim_call_function'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_strwidth'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_vim_strwidth'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_list_runtime_paths'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<QByteArray>, std::false_type>,
        // method 'err_vim_list_runtime_paths'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_change_directory'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_change_directory'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'err_vim_get_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_set_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_set_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_del_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_del_current_line'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_vim_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_vvar'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_vim_get_vvar'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_vim_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_out_write'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_out_write'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_err_write'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_err_write'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_report_error'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_report_error'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_buffers'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<int64_t>, std::false_type>,
        // method 'err_vim_get_buffers'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_current_buffer'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_vim_get_current_buffer'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_set_current_buffer'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_set_current_buffer'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_windows'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<int64_t>, std::false_type>,
        // method 'err_vim_get_windows'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_current_window'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_vim_get_current_window'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_set_current_window'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_set_current_window'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_tabpages'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<int64_t>, std::false_type>,
        // method 'err_vim_get_tabpages'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_current_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_vim_get_current_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_set_current_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_set_current_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_subscribe'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_subscribe'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_unsubscribe'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_vim_unsubscribe'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_name_to_color'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_vim_name_to_color'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_vim_get_color_map'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariantMap, std::false_type>,
        // method 'err_vim_get_color_map'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_get_buffer'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_window_get_buffer'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_get_cursor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'err_window_get_cursor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_set_cursor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_window_set_cursor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_get_height'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_window_get_height'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_set_height'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_window_set_height'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_get_width'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_window_get_width'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_set_width'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_window_set_width'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_window_get_var'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'err_window_get_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'err_window_set_option'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_get_position'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'err_window_get_position'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_get_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'err_window_get_tabpage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'on_window_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'err_window_is_valid'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'handleResponse'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint32, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint64, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'handleResponseError'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint32, std::false_type>,
        QtPrivate::TypeAndForceComplete<quint64, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QVariant &, std::false_type>,
        // method 'nvim_buf_line_count'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_get_line'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_set_line'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'buffer_del_line'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_get_line_slice'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'nvim_buf_get_lines'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'buffer_set_line_slice'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<QByteArray>, std::false_type>,
        // method 'nvim_buf_set_lines'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<QByteArray>, std::false_type>,
        // method 'nvim_buf_get_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_buf_get_changedtick'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_buf_set_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'nvim_buf_del_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'buffer_set_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'buffer_del_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_buf_get_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_buf_set_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'nvim_buf_get_number'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_buf_get_name'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_buf_set_name'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_buf_is_valid'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_insert'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<QByteArray>, std::false_type>,
        // method 'nvim_buf_get_mark'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_buf_add_highlight'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_buf_clear_highlight'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_tabpage_list_wins'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_tabpage_get_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_tabpage_set_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'nvim_tabpage_del_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'tabpage_set_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'tabpage_del_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_tabpage_get_win'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_tabpage_get_number'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_tabpage_is_valid'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_ui_attach'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariantMap, std::false_type>,
        // method 'ui_attach'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'nvim_ui_detach'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_ui_try_resize'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_ui_set_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'nvim_command'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_feedkeys'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'nvim_input'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_replace_termcodes'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'nvim_command_output'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_eval'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_call_function'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariantList, std::false_type>,
        // method 'nvim_strwidth'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_list_runtime_paths'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_set_current_dir'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_get_current_line'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_set_current_line'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_del_current_line'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_get_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_set_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'nvim_del_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_set_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'vim_del_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_get_vvar'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_get_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_set_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'nvim_out_write'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_err_write'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_err_writeln'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_list_bufs'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_get_current_buf'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_set_current_buf'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_list_wins'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_get_current_win'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_set_current_win'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_list_tabpages'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_get_current_tabpage'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_set_current_tabpage'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_subscribe'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_unsubscribe'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_get_color_by_name'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_get_color_map'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_get_mode'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_get_api_info'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'nvim_call_atomic'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariantList, std::false_type>,
        // method 'nvim_win_get_buf'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_win_get_cursor'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_win_set_cursor'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'nvim_win_get_height'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_win_set_height'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_win_get_width'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_win_set_width'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_win_get_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_win_set_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'nvim_win_del_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'window_set_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'window_del_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_win_get_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'nvim_win_set_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'nvim_win_get_position'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_win_get_tabpage'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_win_get_number'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'nvim_win_is_valid'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_line_count'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_get_lines'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'buffer_set_lines'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<QList<QByteArray>, std::false_type>,
        // method 'buffer_get_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'buffer_get_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'buffer_set_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'buffer_get_number'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_get_name'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_set_name'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'buffer_is_valid'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_get_mark'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'buffer_add_highlight'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'buffer_clear_highlight'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'tabpage_get_windows'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'tabpage_get_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'tabpage_get_window'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'tabpage_is_valid'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'ui_detach'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'ui_try_resize'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'vim_command'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_feedkeys'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'vim_input'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_replace_termcodes'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'vim_command_output'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_eval'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_call_function'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariantList, std::false_type>,
        // method 'vim_strwidth'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_list_runtime_paths'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'vim_change_directory'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_get_current_line'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'vim_set_current_line'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_del_current_line'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'vim_get_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_get_vvar'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_get_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_set_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'vim_out_write'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_err_write'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_report_error'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_get_buffers'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'vim_get_current_buffer'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'vim_set_current_buffer'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'vim_get_windows'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'vim_get_current_window'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'vim_set_current_window'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'vim_get_tabpages'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'vim_get_current_tabpage'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'vim_set_current_tabpage'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'vim_subscribe'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_unsubscribe'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_name_to_color'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'vim_get_color_map'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        // method 'window_get_buffer'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'window_get_cursor'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'window_set_cursor'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QPoint, std::false_type>,
        // method 'window_get_height'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'window_set_height'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'window_get_width'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'window_set_width'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'window_get_var'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'window_get_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        // method 'window_set_option'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        QtPrivate::TypeAndForceComplete<QByteArray, std::false_type>,
        QtPrivate::TypeAndForceComplete<QVariant, std::false_type>,
        // method 'window_get_position'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'window_get_tabpage'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>,
        // method 'window_is_valid'
        QtPrivate::TypeAndForceComplete<NeovimQt::MsgpackRequest *, std::false_type>,
        QtPrivate::TypeAndForceComplete<int64_t, std::false_type>
    >,
    nullptr
} };

void NeovimQt::NeovimApi2::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NeovimApi2 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->error((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 1: _t->neovimNotification((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariantList>>(_a[2]))); break;
        case 2: _t->on_nvim_buf_line_count((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 3: _t->err_nvim_buf_line_count((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 4: _t->on_buffer_get_line((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 5: _t->err_buffer_get_line((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 6: _t->on_buffer_set_line(); break;
        case 7: _t->err_buffer_set_line((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 8: _t->on_buffer_del_line(); break;
        case 9: _t->err_buffer_del_line((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 10: _t->on_buffer_get_line_slice((*reinterpret_cast< std::add_pointer_t<QList<QByteArray>>>(_a[1]))); break;
        case 11: _t->err_buffer_get_line_slice((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 12: _t->on_nvim_buf_get_lines((*reinterpret_cast< std::add_pointer_t<QList<QByteArray>>>(_a[1]))); break;
        case 13: _t->err_nvim_buf_get_lines((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 14: _t->on_buffer_set_line_slice(); break;
        case 15: _t->err_buffer_set_line_slice((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 16: _t->on_nvim_buf_set_lines(); break;
        case 17: _t->err_nvim_buf_set_lines((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 18: _t->on_nvim_buf_get_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 19: _t->err_nvim_buf_get_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 20: _t->on_nvim_buf_get_changedtick((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 21: _t->err_nvim_buf_get_changedtick((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 22: _t->on_nvim_buf_set_var(); break;
        case 23: _t->err_nvim_buf_set_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 24: _t->on_nvim_buf_del_var(); break;
        case 25: _t->err_nvim_buf_del_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 26: _t->on_buffer_set_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 27: _t->err_buffer_set_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 28: _t->on_buffer_del_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 29: _t->err_buffer_del_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 30: _t->on_nvim_buf_get_option((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 31: _t->err_nvim_buf_get_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 32: _t->on_nvim_buf_set_option(); break;
        case 33: _t->err_nvim_buf_set_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 34: _t->on_nvim_buf_get_number((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 35: _t->err_nvim_buf_get_number((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 36: _t->on_nvim_buf_get_name((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 37: _t->err_nvim_buf_get_name((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 38: _t->on_nvim_buf_set_name(); break;
        case 39: _t->err_nvim_buf_set_name((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 40: _t->on_nvim_buf_is_valid((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->err_nvim_buf_is_valid((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 42: _t->on_buffer_insert(); break;
        case 43: _t->err_buffer_insert((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 44: _t->on_nvim_buf_get_mark((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 45: _t->err_nvim_buf_get_mark((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 46: _t->on_nvim_buf_add_highlight((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 47: _t->err_nvim_buf_add_highlight((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 48: _t->on_nvim_buf_clear_highlight(); break;
        case 49: _t->err_nvim_buf_clear_highlight((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 50: _t->on_nvim_tabpage_list_wins((*reinterpret_cast< std::add_pointer_t<QList<int64_t>>>(_a[1]))); break;
        case 51: _t->err_nvim_tabpage_list_wins((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 52: _t->on_nvim_tabpage_get_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 53: _t->err_nvim_tabpage_get_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 54: _t->on_nvim_tabpage_set_var(); break;
        case 55: _t->err_nvim_tabpage_set_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 56: _t->on_nvim_tabpage_del_var(); break;
        case 57: _t->err_nvim_tabpage_del_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 58: _t->on_tabpage_set_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 59: _t->err_tabpage_set_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 60: _t->on_tabpage_del_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 61: _t->err_tabpage_del_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 62: _t->on_nvim_tabpage_get_win((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 63: _t->err_nvim_tabpage_get_win((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 64: _t->on_nvim_tabpage_get_number((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 65: _t->err_nvim_tabpage_get_number((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 66: _t->on_nvim_tabpage_is_valid((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 67: _t->err_nvim_tabpage_is_valid((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 68: _t->on_nvim_ui_attach(); break;
        case 69: _t->err_nvim_ui_attach((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 70: _t->on_ui_attach(); break;
        case 71: _t->err_ui_attach((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 72: _t->on_nvim_ui_detach(); break;
        case 73: _t->err_nvim_ui_detach((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 74: _t->on_nvim_ui_try_resize(); break;
        case 75: _t->err_nvim_ui_try_resize((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 76: _t->on_nvim_ui_set_option(); break;
        case 77: _t->err_nvim_ui_set_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 78: _t->on_nvim_command(); break;
        case 79: _t->err_nvim_command((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 80: _t->on_nvim_feedkeys(); break;
        case 81: _t->err_nvim_feedkeys((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 82: _t->on_nvim_input((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 83: _t->err_nvim_input((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 84: _t->on_nvim_replace_termcodes((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 85: _t->err_nvim_replace_termcodes((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 86: _t->on_nvim_command_output((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 87: _t->err_nvim_command_output((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 88: _t->on_nvim_eval((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 89: _t->err_nvim_eval((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 90: _t->on_nvim_call_function((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 91: _t->err_nvim_call_function((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 92: _t->on_nvim_strwidth((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 93: _t->err_nvim_strwidth((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 94: _t->on_nvim_list_runtime_paths((*reinterpret_cast< std::add_pointer_t<QList<QByteArray>>>(_a[1]))); break;
        case 95: _t->err_nvim_list_runtime_paths((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 96: _t->on_nvim_set_current_dir(); break;
        case 97: _t->err_nvim_set_current_dir((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 98: _t->on_nvim_get_current_line((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 99: _t->err_nvim_get_current_line((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 100: _t->on_nvim_set_current_line(); break;
        case 101: _t->err_nvim_set_current_line((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 102: _t->on_nvim_del_current_line(); break;
        case 103: _t->err_nvim_del_current_line((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 104: _t->on_nvim_get_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 105: _t->err_nvim_get_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 106: _t->on_nvim_set_var(); break;
        case 107: _t->err_nvim_set_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 108: _t->on_nvim_del_var(); break;
        case 109: _t->err_nvim_del_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 110: _t->on_vim_set_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 111: _t->err_vim_set_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 112: _t->on_vim_del_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 113: _t->err_vim_del_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 114: _t->on_nvim_get_vvar((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 115: _t->err_nvim_get_vvar((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 116: _t->on_nvim_get_option((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 117: _t->err_nvim_get_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 118: _t->on_nvim_set_option(); break;
        case 119: _t->err_nvim_set_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 120: _t->on_nvim_out_write(); break;
        case 121: _t->err_nvim_out_write((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 122: _t->on_nvim_err_write(); break;
        case 123: _t->err_nvim_err_write((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 124: _t->on_nvim_err_writeln(); break;
        case 125: _t->err_nvim_err_writeln((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 126: _t->on_nvim_list_bufs((*reinterpret_cast< std::add_pointer_t<QList<int64_t>>>(_a[1]))); break;
        case 127: _t->err_nvim_list_bufs((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 128: _t->on_nvim_get_current_buf((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 129: _t->err_nvim_get_current_buf((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 130: _t->on_nvim_set_current_buf(); break;
        case 131: _t->err_nvim_set_current_buf((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 132: _t->on_nvim_list_wins((*reinterpret_cast< std::add_pointer_t<QList<int64_t>>>(_a[1]))); break;
        case 133: _t->err_nvim_list_wins((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 134: _t->on_nvim_get_current_win((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 135: _t->err_nvim_get_current_win((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 136: _t->on_nvim_set_current_win(); break;
        case 137: _t->err_nvim_set_current_win((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 138: _t->on_nvim_list_tabpages((*reinterpret_cast< std::add_pointer_t<QList<int64_t>>>(_a[1]))); break;
        case 139: _t->err_nvim_list_tabpages((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 140: _t->on_nvim_get_current_tabpage((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 141: _t->err_nvim_get_current_tabpage((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 142: _t->on_nvim_set_current_tabpage(); break;
        case 143: _t->err_nvim_set_current_tabpage((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 144: _t->on_nvim_subscribe(); break;
        case 145: _t->err_nvim_subscribe((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 146: _t->on_nvim_unsubscribe(); break;
        case 147: _t->err_nvim_unsubscribe((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 148: _t->on_nvim_get_color_by_name((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 149: _t->err_nvim_get_color_by_name((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 150: _t->on_nvim_get_color_map((*reinterpret_cast< std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 151: _t->err_nvim_get_color_map((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 152: _t->on_nvim_get_mode((*reinterpret_cast< std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 153: _t->err_nvim_get_mode((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 154: _t->on_nvim_get_api_info((*reinterpret_cast< std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 155: _t->err_nvim_get_api_info((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 156: _t->on_nvim_call_atomic((*reinterpret_cast< std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 157: _t->err_nvim_call_atomic((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 158: _t->on_nvim_win_get_buf((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 159: _t->err_nvim_win_get_buf((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 160: _t->on_nvim_win_get_cursor((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 161: _t->err_nvim_win_get_cursor((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 162: _t->on_nvim_win_set_cursor(); break;
        case 163: _t->err_nvim_win_set_cursor((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 164: _t->on_nvim_win_get_height((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 165: _t->err_nvim_win_get_height((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 166: _t->on_nvim_win_set_height(); break;
        case 167: _t->err_nvim_win_set_height((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 168: _t->on_nvim_win_get_width((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 169: _t->err_nvim_win_get_width((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 170: _t->on_nvim_win_set_width(); break;
        case 171: _t->err_nvim_win_set_width((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 172: _t->on_nvim_win_get_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 173: _t->err_nvim_win_get_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 174: _t->on_nvim_win_set_var(); break;
        case 175: _t->err_nvim_win_set_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 176: _t->on_nvim_win_del_var(); break;
        case 177: _t->err_nvim_win_del_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 178: _t->on_window_set_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 179: _t->err_window_set_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 180: _t->on_window_del_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 181: _t->err_window_del_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 182: _t->on_nvim_win_get_option((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 183: _t->err_nvim_win_get_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 184: _t->on_nvim_win_set_option(); break;
        case 185: _t->err_nvim_win_set_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 186: _t->on_nvim_win_get_position((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 187: _t->err_nvim_win_get_position((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 188: _t->on_nvim_win_get_tabpage((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 189: _t->err_nvim_win_get_tabpage((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 190: _t->on_nvim_win_get_number((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 191: _t->err_nvim_win_get_number((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 192: _t->on_nvim_win_is_valid((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 193: _t->err_nvim_win_is_valid((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 194: _t->on_buffer_line_count((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 195: _t->err_buffer_line_count((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 196: _t->on_buffer_get_lines((*reinterpret_cast< std::add_pointer_t<QList<QByteArray>>>(_a[1]))); break;
        case 197: _t->err_buffer_get_lines((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 198: _t->on_buffer_set_lines(); break;
        case 199: _t->err_buffer_set_lines((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 200: _t->on_buffer_get_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 201: _t->err_buffer_get_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 202: _t->on_buffer_get_option((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 203: _t->err_buffer_get_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 204: _t->on_buffer_set_option(); break;
        case 205: _t->err_buffer_set_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 206: _t->on_buffer_get_number((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 207: _t->err_buffer_get_number((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 208: _t->on_buffer_get_name((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 209: _t->err_buffer_get_name((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 210: _t->on_buffer_set_name(); break;
        case 211: _t->err_buffer_set_name((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 212: _t->on_buffer_is_valid((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 213: _t->err_buffer_is_valid((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 214: _t->on_buffer_get_mark((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 215: _t->err_buffer_get_mark((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 216: _t->on_buffer_add_highlight((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 217: _t->err_buffer_add_highlight((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 218: _t->on_buffer_clear_highlight(); break;
        case 219: _t->err_buffer_clear_highlight((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 220: _t->on_tabpage_get_windows((*reinterpret_cast< std::add_pointer_t<QList<int64_t>>>(_a[1]))); break;
        case 221: _t->err_tabpage_get_windows((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 222: _t->on_tabpage_get_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 223: _t->err_tabpage_get_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 224: _t->on_tabpage_get_window((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 225: _t->err_tabpage_get_window((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 226: _t->on_tabpage_is_valid((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 227: _t->err_tabpage_is_valid((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 228: _t->on_ui_detach(); break;
        case 229: _t->err_ui_detach((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 230: _t->on_ui_try_resize((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 231: _t->err_ui_try_resize((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 232: _t->on_vim_command(); break;
        case 233: _t->err_vim_command((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 234: _t->on_vim_feedkeys(); break;
        case 235: _t->err_vim_feedkeys((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 236: _t->on_vim_input((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 237: _t->err_vim_input((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 238: _t->on_vim_replace_termcodes((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 239: _t->err_vim_replace_termcodes((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 240: _t->on_vim_command_output((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 241: _t->err_vim_command_output((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 242: _t->on_vim_eval((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 243: _t->err_vim_eval((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 244: _t->on_vim_call_function((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 245: _t->err_vim_call_function((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 246: _t->on_vim_strwidth((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 247: _t->err_vim_strwidth((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 248: _t->on_vim_list_runtime_paths((*reinterpret_cast< std::add_pointer_t<QList<QByteArray>>>(_a[1]))); break;
        case 249: _t->err_vim_list_runtime_paths((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 250: _t->on_vim_change_directory(); break;
        case 251: _t->err_vim_change_directory((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 252: _t->on_vim_get_current_line((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 253: _t->err_vim_get_current_line((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 254: _t->on_vim_set_current_line(); break;
        case 255: _t->err_vim_set_current_line((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 256: _t->on_vim_del_current_line(); break;
        case 257: _t->err_vim_del_current_line((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 258: _t->on_vim_get_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 259: _t->err_vim_get_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 260: _t->on_vim_get_vvar((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 261: _t->err_vim_get_vvar((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 262: _t->on_vim_get_option((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 263: _t->err_vim_get_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 264: _t->on_vim_set_option(); break;
        case 265: _t->err_vim_set_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 266: _t->on_vim_out_write(); break;
        case 267: _t->err_vim_out_write((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 268: _t->on_vim_err_write(); break;
        case 269: _t->err_vim_err_write((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 270: _t->on_vim_report_error(); break;
        case 271: _t->err_vim_report_error((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 272: _t->on_vim_get_buffers((*reinterpret_cast< std::add_pointer_t<QList<int64_t>>>(_a[1]))); break;
        case 273: _t->err_vim_get_buffers((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 274: _t->on_vim_get_current_buffer((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 275: _t->err_vim_get_current_buffer((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 276: _t->on_vim_set_current_buffer(); break;
        case 277: _t->err_vim_set_current_buffer((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 278: _t->on_vim_get_windows((*reinterpret_cast< std::add_pointer_t<QList<int64_t>>>(_a[1]))); break;
        case 279: _t->err_vim_get_windows((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 280: _t->on_vim_get_current_window((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 281: _t->err_vim_get_current_window((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 282: _t->on_vim_set_current_window(); break;
        case 283: _t->err_vim_set_current_window((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 284: _t->on_vim_get_tabpages((*reinterpret_cast< std::add_pointer_t<QList<int64_t>>>(_a[1]))); break;
        case 285: _t->err_vim_get_tabpages((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 286: _t->on_vim_get_current_tabpage((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 287: _t->err_vim_get_current_tabpage((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 288: _t->on_vim_set_current_tabpage(); break;
        case 289: _t->err_vim_set_current_tabpage((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 290: _t->on_vim_subscribe(); break;
        case 291: _t->err_vim_subscribe((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 292: _t->on_vim_unsubscribe(); break;
        case 293: _t->err_vim_unsubscribe((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 294: _t->on_vim_name_to_color((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 295: _t->err_vim_name_to_color((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 296: _t->on_vim_get_color_map((*reinterpret_cast< std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 297: _t->err_vim_get_color_map((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 298: _t->on_window_get_buffer((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 299: _t->err_window_get_buffer((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 300: _t->on_window_get_cursor((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 301: _t->err_window_get_cursor((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 302: _t->on_window_set_cursor(); break;
        case 303: _t->err_window_set_cursor((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 304: _t->on_window_get_height((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 305: _t->err_window_get_height((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 306: _t->on_window_set_height(); break;
        case 307: _t->err_window_set_height((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 308: _t->on_window_get_width((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 309: _t->err_window_get_width((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 310: _t->on_window_set_width(); break;
        case 311: _t->err_window_set_width((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 312: _t->on_window_get_var((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 313: _t->err_window_get_var((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 314: _t->on_window_get_option((*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 315: _t->err_window_get_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 316: _t->on_window_set_option(); break;
        case 317: _t->err_window_set_option((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 318: _t->on_window_get_position((*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 319: _t->err_window_get_position((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 320: _t->on_window_get_tabpage((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1]))); break;
        case 321: _t->err_window_get_tabpage((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 322: _t->on_window_is_valid((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 323: _t->err_window_is_valid((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 324: _t->handleResponse((*reinterpret_cast< std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<quint64>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 325: _t->handleResponseError((*reinterpret_cast< std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<quint64>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 326: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_line_count((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 327: { NeovimQt::MsgpackRequest* _r = _t->buffer_get_line((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 328: { NeovimQt::MsgpackRequest* _r = _t->buffer_set_line((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 329: { NeovimQt::MsgpackRequest* _r = _t->buffer_del_line((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 330: { NeovimQt::MsgpackRequest* _r = _t->buffer_get_line_slice((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[5])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 331: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_get_lines((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 332: { NeovimQt::MsgpackRequest* _r = _t->buffer_set_line_slice((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast< std::add_pointer_t<QList<QByteArray>>>(_a[6])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 333: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_set_lines((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast< std::add_pointer_t<QList<QByteArray>>>(_a[5])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 334: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_get_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 335: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_get_changedtick((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 336: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_set_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 337: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_del_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 338: { NeovimQt::MsgpackRequest* _r = _t->buffer_set_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 339: { NeovimQt::MsgpackRequest* _r = _t->buffer_del_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 340: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_get_option((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 341: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_set_option((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 342: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_get_number((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 343: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_get_name((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 344: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_set_name((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 345: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_is_valid((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 346: { NeovimQt::MsgpackRequest* _r = _t->buffer_insert((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QList<QByteArray>>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 347: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_get_mark((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 348: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_add_highlight((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[4])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[5])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[6])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 349: { NeovimQt::MsgpackRequest* _r = _t->nvim_buf_clear_highlight((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[4])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 350: { NeovimQt::MsgpackRequest* _r = _t->nvim_tabpage_list_wins((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 351: { NeovimQt::MsgpackRequest* _r = _t->nvim_tabpage_get_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 352: { NeovimQt::MsgpackRequest* _r = _t->nvim_tabpage_set_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 353: { NeovimQt::MsgpackRequest* _r = _t->nvim_tabpage_del_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 354: { NeovimQt::MsgpackRequest* _r = _t->tabpage_set_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 355: { NeovimQt::MsgpackRequest* _r = _t->tabpage_del_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 356: { NeovimQt::MsgpackRequest* _r = _t->nvim_tabpage_get_win((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 357: { NeovimQt::MsgpackRequest* _r = _t->nvim_tabpage_get_number((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 358: { NeovimQt::MsgpackRequest* _r = _t->nvim_tabpage_is_valid((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 359: { NeovimQt::MsgpackRequest* _r = _t->nvim_ui_attach((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariantMap>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 360: { NeovimQt::MsgpackRequest* _r = _t->ui_attach((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 361: { NeovimQt::MsgpackRequest* _r = _t->nvim_ui_detach();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 362: { NeovimQt::MsgpackRequest* _r = _t->nvim_ui_try_resize((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 363: { NeovimQt::MsgpackRequest* _r = _t->nvim_ui_set_option((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 364: { NeovimQt::MsgpackRequest* _r = _t->nvim_command((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 365: { NeovimQt::MsgpackRequest* _r = _t->nvim_feedkeys((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 366: { NeovimQt::MsgpackRequest* _r = _t->nvim_input((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 367: { NeovimQt::MsgpackRequest* _r = _t->nvim_replace_termcodes((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 368: { NeovimQt::MsgpackRequest* _r = _t->nvim_command_output((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 369: { NeovimQt::MsgpackRequest* _r = _t->nvim_eval((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 370: { NeovimQt::MsgpackRequest* _r = _t->nvim_call_function((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariantList>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 371: { NeovimQt::MsgpackRequest* _r = _t->nvim_strwidth((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 372: { NeovimQt::MsgpackRequest* _r = _t->nvim_list_runtime_paths();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 373: { NeovimQt::MsgpackRequest* _r = _t->nvim_set_current_dir((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 374: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_current_line();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 375: { NeovimQt::MsgpackRequest* _r = _t->nvim_set_current_line((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 376: { NeovimQt::MsgpackRequest* _r = _t->nvim_del_current_line();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 377: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_var((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 378: { NeovimQt::MsgpackRequest* _r = _t->nvim_set_var((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 379: { NeovimQt::MsgpackRequest* _r = _t->nvim_del_var((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 380: { NeovimQt::MsgpackRequest* _r = _t->vim_set_var((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 381: { NeovimQt::MsgpackRequest* _r = _t->vim_del_var((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 382: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_vvar((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 383: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_option((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 384: { NeovimQt::MsgpackRequest* _r = _t->nvim_set_option((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 385: { NeovimQt::MsgpackRequest* _r = _t->nvim_out_write((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 386: { NeovimQt::MsgpackRequest* _r = _t->nvim_err_write((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 387: { NeovimQt::MsgpackRequest* _r = _t->nvim_err_writeln((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 388: { NeovimQt::MsgpackRequest* _r = _t->nvim_list_bufs();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 389: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_current_buf();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 390: { NeovimQt::MsgpackRequest* _r = _t->nvim_set_current_buf((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 391: { NeovimQt::MsgpackRequest* _r = _t->nvim_list_wins();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 392: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_current_win();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 393: { NeovimQt::MsgpackRequest* _r = _t->nvim_set_current_win((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 394: { NeovimQt::MsgpackRequest* _r = _t->nvim_list_tabpages();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 395: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_current_tabpage();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 396: { NeovimQt::MsgpackRequest* _r = _t->nvim_set_current_tabpage((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 397: { NeovimQt::MsgpackRequest* _r = _t->nvim_subscribe((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 398: { NeovimQt::MsgpackRequest* _r = _t->nvim_unsubscribe((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 399: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_color_by_name((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 400: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_color_map();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 401: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_mode();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 402: { NeovimQt::MsgpackRequest* _r = _t->nvim_get_api_info();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 403: { NeovimQt::MsgpackRequest* _r = _t->nvim_call_atomic((*reinterpret_cast< std::add_pointer_t<QVariantList>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 404: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_get_buf((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 405: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_get_cursor((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 406: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_set_cursor((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 407: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_get_height((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 408: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_set_height((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 409: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_get_width((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 410: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_set_width((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 411: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_get_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 412: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_set_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 413: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_del_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 414: { NeovimQt::MsgpackRequest* _r = _t->window_set_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 415: { NeovimQt::MsgpackRequest* _r = _t->window_del_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 416: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_get_option((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 417: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_set_option((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 418: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_get_position((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 419: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_get_tabpage((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 420: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_get_number((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 421: { NeovimQt::MsgpackRequest* _r = _t->nvim_win_is_valid((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 422: { NeovimQt::MsgpackRequest* _r = _t->buffer_line_count((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 423: { NeovimQt::MsgpackRequest* _r = _t->buffer_get_lines((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 424: { NeovimQt::MsgpackRequest* _r = _t->buffer_set_lines((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast< std::add_pointer_t<QList<QByteArray>>>(_a[5])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 425: { NeovimQt::MsgpackRequest* _r = _t->buffer_get_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 426: { NeovimQt::MsgpackRequest* _r = _t->buffer_get_option((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 427: { NeovimQt::MsgpackRequest* _r = _t->buffer_set_option((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 428: { NeovimQt::MsgpackRequest* _r = _t->buffer_get_number((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 429: { NeovimQt::MsgpackRequest* _r = _t->buffer_get_name((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 430: { NeovimQt::MsgpackRequest* _r = _t->buffer_set_name((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 431: { NeovimQt::MsgpackRequest* _r = _t->buffer_is_valid((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 432: { NeovimQt::MsgpackRequest* _r = _t->buffer_get_mark((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 433: { NeovimQt::MsgpackRequest* _r = _t->buffer_add_highlight((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[4])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[5])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[6])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 434: { NeovimQt::MsgpackRequest* _r = _t->buffer_clear_highlight((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[4])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 435: { NeovimQt::MsgpackRequest* _r = _t->tabpage_get_windows((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 436: { NeovimQt::MsgpackRequest* _r = _t->tabpage_get_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 437: { NeovimQt::MsgpackRequest* _r = _t->tabpage_get_window((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 438: { NeovimQt::MsgpackRequest* _r = _t->tabpage_is_valid((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 439: { NeovimQt::MsgpackRequest* _r = _t->ui_detach();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 440: { NeovimQt::MsgpackRequest* _r = _t->ui_try_resize((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 441: { NeovimQt::MsgpackRequest* _r = _t->vim_command((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 442: { NeovimQt::MsgpackRequest* _r = _t->vim_feedkeys((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 443: { NeovimQt::MsgpackRequest* _r = _t->vim_input((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 444: { NeovimQt::MsgpackRequest* _r = _t->vim_replace_termcodes((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast< std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 445: { NeovimQt::MsgpackRequest* _r = _t->vim_command_output((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 446: { NeovimQt::MsgpackRequest* _r = _t->vim_eval((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 447: { NeovimQt::MsgpackRequest* _r = _t->vim_call_function((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariantList>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 448: { NeovimQt::MsgpackRequest* _r = _t->vim_strwidth((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 449: { NeovimQt::MsgpackRequest* _r = _t->vim_list_runtime_paths();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 450: { NeovimQt::MsgpackRequest* _r = _t->vim_change_directory((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 451: { NeovimQt::MsgpackRequest* _r = _t->vim_get_current_line();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 452: { NeovimQt::MsgpackRequest* _r = _t->vim_set_current_line((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 453: { NeovimQt::MsgpackRequest* _r = _t->vim_del_current_line();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 454: { NeovimQt::MsgpackRequest* _r = _t->vim_get_var((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 455: { NeovimQt::MsgpackRequest* _r = _t->vim_get_vvar((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 456: { NeovimQt::MsgpackRequest* _r = _t->vim_get_option((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 457: { NeovimQt::MsgpackRequest* _r = _t->vim_set_option((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 458: { NeovimQt::MsgpackRequest* _r = _t->vim_out_write((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 459: { NeovimQt::MsgpackRequest* _r = _t->vim_err_write((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 460: { NeovimQt::MsgpackRequest* _r = _t->vim_report_error((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 461: { NeovimQt::MsgpackRequest* _r = _t->vim_get_buffers();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 462: { NeovimQt::MsgpackRequest* _r = _t->vim_get_current_buffer();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 463: { NeovimQt::MsgpackRequest* _r = _t->vim_set_current_buffer((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 464: { NeovimQt::MsgpackRequest* _r = _t->vim_get_windows();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 465: { NeovimQt::MsgpackRequest* _r = _t->vim_get_current_window();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 466: { NeovimQt::MsgpackRequest* _r = _t->vim_set_current_window((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 467: { NeovimQt::MsgpackRequest* _r = _t->vim_get_tabpages();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 468: { NeovimQt::MsgpackRequest* _r = _t->vim_get_current_tabpage();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 469: { NeovimQt::MsgpackRequest* _r = _t->vim_set_current_tabpage((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 470: { NeovimQt::MsgpackRequest* _r = _t->vim_subscribe((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 471: { NeovimQt::MsgpackRequest* _r = _t->vim_unsubscribe((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 472: { NeovimQt::MsgpackRequest* _r = _t->vim_name_to_color((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 473: { NeovimQt::MsgpackRequest* _r = _t->vim_get_color_map();
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 474: { NeovimQt::MsgpackRequest* _r = _t->window_get_buffer((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 475: { NeovimQt::MsgpackRequest* _r = _t->window_get_cursor((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 476: { NeovimQt::MsgpackRequest* _r = _t->window_set_cursor((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QPoint>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 477: { NeovimQt::MsgpackRequest* _r = _t->window_get_height((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 478: { NeovimQt::MsgpackRequest* _r = _t->window_set_height((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 479: { NeovimQt::MsgpackRequest* _r = _t->window_get_width((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 480: { NeovimQt::MsgpackRequest* _r = _t->window_set_width((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 481: { NeovimQt::MsgpackRequest* _r = _t->window_get_var((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 482: { NeovimQt::MsgpackRequest* _r = _t->window_get_option((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 483: { NeovimQt::MsgpackRequest* _r = _t->window_set_option((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 484: { NeovimQt::MsgpackRequest* _r = _t->window_get_position((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 485: { NeovimQt::MsgpackRequest* _r = _t->window_get_tabpage((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        case 486: { NeovimQt::MsgpackRequest* _r = _t->window_is_valid((*reinterpret_cast< std::add_pointer_t<int64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast< NeovimQt::MsgpackRequest**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::error; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QByteArray & , const QVariantList & );
            if (_q_method_type _q_method = &NeovimApi2::neovimNotification; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_line_count; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_line_count; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QByteArray );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_get_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_get_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 5;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_set_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 6;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_set_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 7;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_del_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 8;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_del_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 9;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<QByteArray> );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_get_line_slice; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 10;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_get_line_slice; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 11;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<QByteArray> );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_get_lines; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 12;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_get_lines; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 13;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_set_line_slice; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 14;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_set_line_slice; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 15;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_set_lines; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 16;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_set_lines; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 17;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 18;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 19;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_get_changedtick; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 20;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_get_changedtick; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 21;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 22;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 23;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 24;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 25;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 26;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 27;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 28;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 29;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 30;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 31;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 32;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 33;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_get_number; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 34;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_get_number; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 35;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QByteArray );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_get_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 36;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_get_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 37;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_set_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 38;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_set_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 39;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(bool );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 40;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 41;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_insert; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 42;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_insert; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 43;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QPoint );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_get_mark; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 44;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_get_mark; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 45;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_add_highlight; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 46;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_add_highlight; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 47;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_buf_clear_highlight; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 48;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_buf_clear_highlight; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 49;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<int64_t> );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_tabpage_list_wins; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 50;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_tabpage_list_wins; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 51;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_tabpage_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 52;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_tabpage_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 53;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_tabpage_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 54;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_tabpage_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 55;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_tabpage_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 56;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_tabpage_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 57;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_tabpage_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 58;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_tabpage_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 59;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_tabpage_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 60;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_tabpage_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 61;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_tabpage_get_win; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 62;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_tabpage_get_win; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 63;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_tabpage_get_number; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 64;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_tabpage_get_number; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 65;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(bool );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_tabpage_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 66;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_tabpage_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 67;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_ui_attach; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 68;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_ui_attach; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 69;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_ui_attach; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 70;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_ui_attach; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 71;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_ui_detach; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 72;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_ui_detach; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 73;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_ui_try_resize; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 74;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_ui_try_resize; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 75;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_ui_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 76;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_ui_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 77;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_command; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 78;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_command; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 79;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_feedkeys; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 80;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_feedkeys; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 81;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_input; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 82;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_input; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 83;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QByteArray );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_replace_termcodes; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 84;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_replace_termcodes; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 85;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QByteArray );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_command_output; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 86;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_command_output; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 87;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_eval; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 88;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_eval; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 89;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_call_function; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 90;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_call_function; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 91;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_strwidth; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 92;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_strwidth; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 93;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<QByteArray> );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_list_runtime_paths; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 94;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_list_runtime_paths; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 95;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_set_current_dir; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 96;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_set_current_dir; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 97;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QByteArray );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 98;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 99;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_set_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 100;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_set_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 101;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_del_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 102;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_del_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 103;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 104;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 105;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 106;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 107;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 108;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 109;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 110;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 111;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 112;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 113;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_vvar; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 114;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_vvar; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 115;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 116;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 117;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 118;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 119;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_out_write; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 120;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_out_write; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 121;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_err_write; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 122;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_err_write; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 123;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_err_writeln; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 124;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_err_writeln; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 125;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<int64_t> );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_list_bufs; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 126;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_list_bufs; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 127;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_current_buf; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 128;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_current_buf; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 129;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_set_current_buf; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 130;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_set_current_buf; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 131;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<int64_t> );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_list_wins; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 132;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_list_wins; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 133;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_current_win; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 134;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_current_win; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 135;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_set_current_win; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 136;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_set_current_win; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 137;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<int64_t> );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_list_tabpages; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 138;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_list_tabpages; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 139;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_current_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 140;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_current_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 141;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_set_current_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 142;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_set_current_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 143;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_subscribe; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 144;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_subscribe; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 145;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_unsubscribe; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 146;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_unsubscribe; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 147;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_color_by_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 148;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_color_by_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 149;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariantMap );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_color_map; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 150;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_color_map; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 151;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariantMap );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_mode; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 152;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_mode; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 153;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariantList );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_get_api_info; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 154;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_get_api_info; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 155;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariantList );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_call_atomic; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 156;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_call_atomic; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 157;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_get_buf; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 158;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_get_buf; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 159;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QPoint );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_get_cursor; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 160;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_get_cursor; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 161;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_set_cursor; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 162;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_set_cursor; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 163;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_get_height; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 164;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_get_height; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 165;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_set_height; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 166;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_set_height; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 167;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_get_width; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 168;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_get_width; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 169;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_set_width; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 170;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_set_width; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 171;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 172;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 173;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 174;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 175;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 176;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 177;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_window_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 178;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_set_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 179;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_window_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 180;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_del_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 181;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 182;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 183;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 184;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 185;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QPoint );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_get_position; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 186;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_get_position; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 187;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_get_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 188;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_get_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 189;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_get_number; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 190;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_get_number; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 191;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(bool );
            if (_q_method_type _q_method = &NeovimApi2::on_nvim_win_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 192;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_nvim_win_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 193;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_line_count; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 194;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_line_count; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 195;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<QByteArray> );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_get_lines; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 196;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_get_lines; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 197;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_set_lines; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 198;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_set_lines; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 199;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 200;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 201;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 202;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 203;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 204;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 205;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_get_number; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 206;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_get_number; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 207;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QByteArray );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_get_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 208;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_get_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 209;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_set_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 210;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_set_name; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 211;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(bool );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 212;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 213;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QPoint );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_get_mark; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 214;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_get_mark; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 215;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_add_highlight; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 216;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_add_highlight; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 217;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_buffer_clear_highlight; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 218;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_buffer_clear_highlight; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 219;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<int64_t> );
            if (_q_method_type _q_method = &NeovimApi2::on_tabpage_get_windows; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 220;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_tabpage_get_windows; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 221;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_tabpage_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 222;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_tabpage_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 223;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_tabpage_get_window; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 224;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_tabpage_get_window; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 225;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(bool );
            if (_q_method_type _q_method = &NeovimApi2::on_tabpage_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 226;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_tabpage_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 227;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_ui_detach; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 228;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_ui_detach; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 229;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_ui_try_resize; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 230;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_ui_try_resize; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 231;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_command; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 232;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_command; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 233;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_feedkeys; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 234;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_feedkeys; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 235;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_input; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 236;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_input; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 237;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QByteArray );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_replace_termcodes; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 238;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_replace_termcodes; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 239;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QByteArray );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_command_output; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 240;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_command_output; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 241;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_eval; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 242;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_eval; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 243;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_call_function; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 244;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_call_function; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 245;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_strwidth; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 246;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_strwidth; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 247;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<QByteArray> );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_list_runtime_paths; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 248;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_list_runtime_paths; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 249;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_change_directory; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 250;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_change_directory; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 251;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QByteArray );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 252;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 253;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_set_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 254;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_set_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 255;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_del_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 256;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_del_current_line; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 257;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 258;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 259;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_vvar; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 260;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_vvar; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 261;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 262;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 263;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 264;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 265;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_out_write; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 266;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_out_write; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 267;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_err_write; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 268;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_err_write; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 269;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_report_error; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 270;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_report_error; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 271;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<int64_t> );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_buffers; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 272;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_buffers; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 273;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_current_buffer; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 274;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_current_buffer; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 275;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_set_current_buffer; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 276;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_set_current_buffer; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 277;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<int64_t> );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_windows; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 278;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_windows; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 279;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_current_window; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 280;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_current_window; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 281;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_set_current_window; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 282;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_set_current_window; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 283;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QList<int64_t> );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_tabpages; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 284;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_tabpages; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 285;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_current_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 286;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_current_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 287;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_set_current_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 288;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_set_current_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 289;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_subscribe; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 290;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_subscribe; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 291;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_vim_unsubscribe; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 292;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_unsubscribe; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 293;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_name_to_color; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 294;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_name_to_color; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 295;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariantMap );
            if (_q_method_type _q_method = &NeovimApi2::on_vim_get_color_map; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 296;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_vim_get_color_map; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 297;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_window_get_buffer; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 298;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_get_buffer; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 299;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QPoint );
            if (_q_method_type _q_method = &NeovimApi2::on_window_get_cursor; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 300;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_get_cursor; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 301;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_window_set_cursor; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 302;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_set_cursor; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 303;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_window_get_height; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 304;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_get_height; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 305;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_window_set_height; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 306;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_set_height; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 307;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_window_get_width; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 308;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_get_width; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 309;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_window_set_width; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 310;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_set_width; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 311;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_window_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 312;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_get_var; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 313;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QVariant );
            if (_q_method_type _q_method = &NeovimApi2::on_window_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 314;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_get_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 315;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)();
            if (_q_method_type _q_method = &NeovimApi2::on_window_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 316;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_set_option; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 317;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(QPoint );
            if (_q_method_type _q_method = &NeovimApi2::on_window_get_position; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 318;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_get_position; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 319;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(int64_t );
            if (_q_method_type _q_method = &NeovimApi2::on_window_get_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 320;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_get_tabpage; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 321;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(bool );
            if (_q_method_type _q_method = &NeovimApi2::on_window_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 322;
                return;
            }
        }
        {
            using _q_method_type = void (NeovimApi2::*)(const QString & , const QVariant & );
            if (_q_method_type _q_method = &NeovimApi2::err_window_is_valid; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 323;
                return;
            }
        }
    }
}

const QMetaObject *NeovimQt::NeovimApi2::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NeovimQt::NeovimApi2::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN8NeovimQt10NeovimApi2E.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int NeovimQt::NeovimApi2::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 487)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 487;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 487)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 487;
    }
    return _id;
}

// SIGNAL 0
void NeovimQt::NeovimApi2::error(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void NeovimQt::NeovimApi2::neovimNotification(const QByteArray & _t1, const QVariantList & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void NeovimQt::NeovimApi2::on_nvim_buf_line_count(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void NeovimQt::NeovimApi2::err_nvim_buf_line_count(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void NeovimQt::NeovimApi2::on_buffer_get_line(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void NeovimQt::NeovimApi2::err_buffer_get_line(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void NeovimQt::NeovimApi2::on_buffer_set_line()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void NeovimQt::NeovimApi2::err_buffer_set_line(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void NeovimQt::NeovimApi2::on_buffer_del_line()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void NeovimQt::NeovimApi2::err_buffer_del_line(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void NeovimQt::NeovimApi2::on_buffer_get_line_slice(QList<QByteArray> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void NeovimQt::NeovimApi2::err_buffer_get_line_slice(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void NeovimQt::NeovimApi2::on_nvim_buf_get_lines(QList<QByteArray> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void NeovimQt::NeovimApi2::err_nvim_buf_get_lines(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void NeovimQt::NeovimApi2::on_buffer_set_line_slice()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void NeovimQt::NeovimApi2::err_buffer_set_line_slice(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void NeovimQt::NeovimApi2::on_nvim_buf_set_lines()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void NeovimQt::NeovimApi2::err_nvim_buf_set_lines(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void NeovimQt::NeovimApi2::on_nvim_buf_get_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void NeovimQt::NeovimApi2::err_nvim_buf_get_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void NeovimQt::NeovimApi2::on_nvim_buf_get_changedtick(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void NeovimQt::NeovimApi2::err_nvim_buf_get_changedtick(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}

// SIGNAL 22
void NeovimQt::NeovimApi2::on_nvim_buf_set_var()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void NeovimQt::NeovimApi2::err_nvim_buf_set_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 23, _a);
}

// SIGNAL 24
void NeovimQt::NeovimApi2::on_nvim_buf_del_var()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void NeovimQt::NeovimApi2::err_nvim_buf_del_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 25, _a);
}

// SIGNAL 26
void NeovimQt::NeovimApi2::on_buffer_set_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 26, _a);
}

// SIGNAL 27
void NeovimQt::NeovimApi2::err_buffer_set_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 27, _a);
}

// SIGNAL 28
void NeovimQt::NeovimApi2::on_buffer_del_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 28, _a);
}

// SIGNAL 29
void NeovimQt::NeovimApi2::err_buffer_del_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 29, _a);
}

// SIGNAL 30
void NeovimQt::NeovimApi2::on_nvim_buf_get_option(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 30, _a);
}

// SIGNAL 31
void NeovimQt::NeovimApi2::err_nvim_buf_get_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 31, _a);
}

// SIGNAL 32
void NeovimQt::NeovimApi2::on_nvim_buf_set_option()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}

// SIGNAL 33
void NeovimQt::NeovimApi2::err_nvim_buf_set_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 33, _a);
}

// SIGNAL 34
void NeovimQt::NeovimApi2::on_nvim_buf_get_number(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 34, _a);
}

// SIGNAL 35
void NeovimQt::NeovimApi2::err_nvim_buf_get_number(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 35, _a);
}

// SIGNAL 36
void NeovimQt::NeovimApi2::on_nvim_buf_get_name(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 36, _a);
}

// SIGNAL 37
void NeovimQt::NeovimApi2::err_nvim_buf_get_name(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 37, _a);
}

// SIGNAL 38
void NeovimQt::NeovimApi2::on_nvim_buf_set_name()
{
    QMetaObject::activate(this, &staticMetaObject, 38, nullptr);
}

// SIGNAL 39
void NeovimQt::NeovimApi2::err_nvim_buf_set_name(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 39, _a);
}

// SIGNAL 40
void NeovimQt::NeovimApi2::on_nvim_buf_is_valid(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 40, _a);
}

// SIGNAL 41
void NeovimQt::NeovimApi2::err_nvim_buf_is_valid(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 41, _a);
}

// SIGNAL 42
void NeovimQt::NeovimApi2::on_buffer_insert()
{
    QMetaObject::activate(this, &staticMetaObject, 42, nullptr);
}

// SIGNAL 43
void NeovimQt::NeovimApi2::err_buffer_insert(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 43, _a);
}

// SIGNAL 44
void NeovimQt::NeovimApi2::on_nvim_buf_get_mark(QPoint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 44, _a);
}

// SIGNAL 45
void NeovimQt::NeovimApi2::err_nvim_buf_get_mark(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 45, _a);
}

// SIGNAL 46
void NeovimQt::NeovimApi2::on_nvim_buf_add_highlight(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 46, _a);
}

// SIGNAL 47
void NeovimQt::NeovimApi2::err_nvim_buf_add_highlight(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 47, _a);
}

// SIGNAL 48
void NeovimQt::NeovimApi2::on_nvim_buf_clear_highlight()
{
    QMetaObject::activate(this, &staticMetaObject, 48, nullptr);
}

// SIGNAL 49
void NeovimQt::NeovimApi2::err_nvim_buf_clear_highlight(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 49, _a);
}

// SIGNAL 50
void NeovimQt::NeovimApi2::on_nvim_tabpage_list_wins(QList<int64_t> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 50, _a);
}

// SIGNAL 51
void NeovimQt::NeovimApi2::err_nvim_tabpage_list_wins(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 51, _a);
}

// SIGNAL 52
void NeovimQt::NeovimApi2::on_nvim_tabpage_get_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 52, _a);
}

// SIGNAL 53
void NeovimQt::NeovimApi2::err_nvim_tabpage_get_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 53, _a);
}

// SIGNAL 54
void NeovimQt::NeovimApi2::on_nvim_tabpage_set_var()
{
    QMetaObject::activate(this, &staticMetaObject, 54, nullptr);
}

// SIGNAL 55
void NeovimQt::NeovimApi2::err_nvim_tabpage_set_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 55, _a);
}

// SIGNAL 56
void NeovimQt::NeovimApi2::on_nvim_tabpage_del_var()
{
    QMetaObject::activate(this, &staticMetaObject, 56, nullptr);
}

// SIGNAL 57
void NeovimQt::NeovimApi2::err_nvim_tabpage_del_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 57, _a);
}

// SIGNAL 58
void NeovimQt::NeovimApi2::on_tabpage_set_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 58, _a);
}

// SIGNAL 59
void NeovimQt::NeovimApi2::err_tabpage_set_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 59, _a);
}

// SIGNAL 60
void NeovimQt::NeovimApi2::on_tabpage_del_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 60, _a);
}

// SIGNAL 61
void NeovimQt::NeovimApi2::err_tabpage_del_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 61, _a);
}

// SIGNAL 62
void NeovimQt::NeovimApi2::on_nvim_tabpage_get_win(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 62, _a);
}

// SIGNAL 63
void NeovimQt::NeovimApi2::err_nvim_tabpage_get_win(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 63, _a);
}

// SIGNAL 64
void NeovimQt::NeovimApi2::on_nvim_tabpage_get_number(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 64, _a);
}

// SIGNAL 65
void NeovimQt::NeovimApi2::err_nvim_tabpage_get_number(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 65, _a);
}

// SIGNAL 66
void NeovimQt::NeovimApi2::on_nvim_tabpage_is_valid(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 66, _a);
}

// SIGNAL 67
void NeovimQt::NeovimApi2::err_nvim_tabpage_is_valid(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 67, _a);
}

// SIGNAL 68
void NeovimQt::NeovimApi2::on_nvim_ui_attach()
{
    QMetaObject::activate(this, &staticMetaObject, 68, nullptr);
}

// SIGNAL 69
void NeovimQt::NeovimApi2::err_nvim_ui_attach(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 69, _a);
}

// SIGNAL 70
void NeovimQt::NeovimApi2::on_ui_attach()
{
    QMetaObject::activate(this, &staticMetaObject, 70, nullptr);
}

// SIGNAL 71
void NeovimQt::NeovimApi2::err_ui_attach(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 71, _a);
}

// SIGNAL 72
void NeovimQt::NeovimApi2::on_nvim_ui_detach()
{
    QMetaObject::activate(this, &staticMetaObject, 72, nullptr);
}

// SIGNAL 73
void NeovimQt::NeovimApi2::err_nvim_ui_detach(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 73, _a);
}

// SIGNAL 74
void NeovimQt::NeovimApi2::on_nvim_ui_try_resize()
{
    QMetaObject::activate(this, &staticMetaObject, 74, nullptr);
}

// SIGNAL 75
void NeovimQt::NeovimApi2::err_nvim_ui_try_resize(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 75, _a);
}

// SIGNAL 76
void NeovimQt::NeovimApi2::on_nvim_ui_set_option()
{
    QMetaObject::activate(this, &staticMetaObject, 76, nullptr);
}

// SIGNAL 77
void NeovimQt::NeovimApi2::err_nvim_ui_set_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 77, _a);
}

// SIGNAL 78
void NeovimQt::NeovimApi2::on_nvim_command()
{
    QMetaObject::activate(this, &staticMetaObject, 78, nullptr);
}

// SIGNAL 79
void NeovimQt::NeovimApi2::err_nvim_command(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 79, _a);
}

// SIGNAL 80
void NeovimQt::NeovimApi2::on_nvim_feedkeys()
{
    QMetaObject::activate(this, &staticMetaObject, 80, nullptr);
}

// SIGNAL 81
void NeovimQt::NeovimApi2::err_nvim_feedkeys(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 81, _a);
}

// SIGNAL 82
void NeovimQt::NeovimApi2::on_nvim_input(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 82, _a);
}

// SIGNAL 83
void NeovimQt::NeovimApi2::err_nvim_input(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 83, _a);
}

// SIGNAL 84
void NeovimQt::NeovimApi2::on_nvim_replace_termcodes(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 84, _a);
}

// SIGNAL 85
void NeovimQt::NeovimApi2::err_nvim_replace_termcodes(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 85, _a);
}

// SIGNAL 86
void NeovimQt::NeovimApi2::on_nvim_command_output(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 86, _a);
}

// SIGNAL 87
void NeovimQt::NeovimApi2::err_nvim_command_output(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 87, _a);
}

// SIGNAL 88
void NeovimQt::NeovimApi2::on_nvim_eval(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 88, _a);
}

// SIGNAL 89
void NeovimQt::NeovimApi2::err_nvim_eval(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 89, _a);
}

// SIGNAL 90
void NeovimQt::NeovimApi2::on_nvim_call_function(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 90, _a);
}

// SIGNAL 91
void NeovimQt::NeovimApi2::err_nvim_call_function(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 91, _a);
}

// SIGNAL 92
void NeovimQt::NeovimApi2::on_nvim_strwidth(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 92, _a);
}

// SIGNAL 93
void NeovimQt::NeovimApi2::err_nvim_strwidth(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 93, _a);
}

// SIGNAL 94
void NeovimQt::NeovimApi2::on_nvim_list_runtime_paths(QList<QByteArray> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 94, _a);
}

// SIGNAL 95
void NeovimQt::NeovimApi2::err_nvim_list_runtime_paths(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 95, _a);
}

// SIGNAL 96
void NeovimQt::NeovimApi2::on_nvim_set_current_dir()
{
    QMetaObject::activate(this, &staticMetaObject, 96, nullptr);
}

// SIGNAL 97
void NeovimQt::NeovimApi2::err_nvim_set_current_dir(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 97, _a);
}

// SIGNAL 98
void NeovimQt::NeovimApi2::on_nvim_get_current_line(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 98, _a);
}

// SIGNAL 99
void NeovimQt::NeovimApi2::err_nvim_get_current_line(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 99, _a);
}

// SIGNAL 100
void NeovimQt::NeovimApi2::on_nvim_set_current_line()
{
    QMetaObject::activate(this, &staticMetaObject, 100, nullptr);
}

// SIGNAL 101
void NeovimQt::NeovimApi2::err_nvim_set_current_line(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 101, _a);
}

// SIGNAL 102
void NeovimQt::NeovimApi2::on_nvim_del_current_line()
{
    QMetaObject::activate(this, &staticMetaObject, 102, nullptr);
}

// SIGNAL 103
void NeovimQt::NeovimApi2::err_nvim_del_current_line(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 103, _a);
}

// SIGNAL 104
void NeovimQt::NeovimApi2::on_nvim_get_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 104, _a);
}

// SIGNAL 105
void NeovimQt::NeovimApi2::err_nvim_get_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 105, _a);
}

// SIGNAL 106
void NeovimQt::NeovimApi2::on_nvim_set_var()
{
    QMetaObject::activate(this, &staticMetaObject, 106, nullptr);
}

// SIGNAL 107
void NeovimQt::NeovimApi2::err_nvim_set_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 107, _a);
}

// SIGNAL 108
void NeovimQt::NeovimApi2::on_nvim_del_var()
{
    QMetaObject::activate(this, &staticMetaObject, 108, nullptr);
}

// SIGNAL 109
void NeovimQt::NeovimApi2::err_nvim_del_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 109, _a);
}

// SIGNAL 110
void NeovimQt::NeovimApi2::on_vim_set_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 110, _a);
}

// SIGNAL 111
void NeovimQt::NeovimApi2::err_vim_set_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 111, _a);
}

// SIGNAL 112
void NeovimQt::NeovimApi2::on_vim_del_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 112, _a);
}

// SIGNAL 113
void NeovimQt::NeovimApi2::err_vim_del_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 113, _a);
}

// SIGNAL 114
void NeovimQt::NeovimApi2::on_nvim_get_vvar(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 114, _a);
}

// SIGNAL 115
void NeovimQt::NeovimApi2::err_nvim_get_vvar(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 115, _a);
}

// SIGNAL 116
void NeovimQt::NeovimApi2::on_nvim_get_option(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 116, _a);
}

// SIGNAL 117
void NeovimQt::NeovimApi2::err_nvim_get_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 117, _a);
}

// SIGNAL 118
void NeovimQt::NeovimApi2::on_nvim_set_option()
{
    QMetaObject::activate(this, &staticMetaObject, 118, nullptr);
}

// SIGNAL 119
void NeovimQt::NeovimApi2::err_nvim_set_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 119, _a);
}

// SIGNAL 120
void NeovimQt::NeovimApi2::on_nvim_out_write()
{
    QMetaObject::activate(this, &staticMetaObject, 120, nullptr);
}

// SIGNAL 121
void NeovimQt::NeovimApi2::err_nvim_out_write(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 121, _a);
}

// SIGNAL 122
void NeovimQt::NeovimApi2::on_nvim_err_write()
{
    QMetaObject::activate(this, &staticMetaObject, 122, nullptr);
}

// SIGNAL 123
void NeovimQt::NeovimApi2::err_nvim_err_write(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 123, _a);
}

// SIGNAL 124
void NeovimQt::NeovimApi2::on_nvim_err_writeln()
{
    QMetaObject::activate(this, &staticMetaObject, 124, nullptr);
}

// SIGNAL 125
void NeovimQt::NeovimApi2::err_nvim_err_writeln(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 125, _a);
}

// SIGNAL 126
void NeovimQt::NeovimApi2::on_nvim_list_bufs(QList<int64_t> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 126, _a);
}

// SIGNAL 127
void NeovimQt::NeovimApi2::err_nvim_list_bufs(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 127, _a);
}

// SIGNAL 128
void NeovimQt::NeovimApi2::on_nvim_get_current_buf(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 128, _a);
}

// SIGNAL 129
void NeovimQt::NeovimApi2::err_nvim_get_current_buf(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 129, _a);
}

// SIGNAL 130
void NeovimQt::NeovimApi2::on_nvim_set_current_buf()
{
    QMetaObject::activate(this, &staticMetaObject, 130, nullptr);
}

// SIGNAL 131
void NeovimQt::NeovimApi2::err_nvim_set_current_buf(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 131, _a);
}

// SIGNAL 132
void NeovimQt::NeovimApi2::on_nvim_list_wins(QList<int64_t> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 132, _a);
}

// SIGNAL 133
void NeovimQt::NeovimApi2::err_nvim_list_wins(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 133, _a);
}

// SIGNAL 134
void NeovimQt::NeovimApi2::on_nvim_get_current_win(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 134, _a);
}

// SIGNAL 135
void NeovimQt::NeovimApi2::err_nvim_get_current_win(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 135, _a);
}

// SIGNAL 136
void NeovimQt::NeovimApi2::on_nvim_set_current_win()
{
    QMetaObject::activate(this, &staticMetaObject, 136, nullptr);
}

// SIGNAL 137
void NeovimQt::NeovimApi2::err_nvim_set_current_win(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 137, _a);
}

// SIGNAL 138
void NeovimQt::NeovimApi2::on_nvim_list_tabpages(QList<int64_t> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 138, _a);
}

// SIGNAL 139
void NeovimQt::NeovimApi2::err_nvim_list_tabpages(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 139, _a);
}

// SIGNAL 140
void NeovimQt::NeovimApi2::on_nvim_get_current_tabpage(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 140, _a);
}

// SIGNAL 141
void NeovimQt::NeovimApi2::err_nvim_get_current_tabpage(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 141, _a);
}

// SIGNAL 142
void NeovimQt::NeovimApi2::on_nvim_set_current_tabpage()
{
    QMetaObject::activate(this, &staticMetaObject, 142, nullptr);
}

// SIGNAL 143
void NeovimQt::NeovimApi2::err_nvim_set_current_tabpage(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 143, _a);
}

// SIGNAL 144
void NeovimQt::NeovimApi2::on_nvim_subscribe()
{
    QMetaObject::activate(this, &staticMetaObject, 144, nullptr);
}

// SIGNAL 145
void NeovimQt::NeovimApi2::err_nvim_subscribe(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 145, _a);
}

// SIGNAL 146
void NeovimQt::NeovimApi2::on_nvim_unsubscribe()
{
    QMetaObject::activate(this, &staticMetaObject, 146, nullptr);
}

// SIGNAL 147
void NeovimQt::NeovimApi2::err_nvim_unsubscribe(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 147, _a);
}

// SIGNAL 148
void NeovimQt::NeovimApi2::on_nvim_get_color_by_name(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 148, _a);
}

// SIGNAL 149
void NeovimQt::NeovimApi2::err_nvim_get_color_by_name(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 149, _a);
}

// SIGNAL 150
void NeovimQt::NeovimApi2::on_nvim_get_color_map(QVariantMap _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 150, _a);
}

// SIGNAL 151
void NeovimQt::NeovimApi2::err_nvim_get_color_map(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 151, _a);
}

// SIGNAL 152
void NeovimQt::NeovimApi2::on_nvim_get_mode(QVariantMap _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 152, _a);
}

// SIGNAL 153
void NeovimQt::NeovimApi2::err_nvim_get_mode(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 153, _a);
}

// SIGNAL 154
void NeovimQt::NeovimApi2::on_nvim_get_api_info(QVariantList _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 154, _a);
}

// SIGNAL 155
void NeovimQt::NeovimApi2::err_nvim_get_api_info(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 155, _a);
}

// SIGNAL 156
void NeovimQt::NeovimApi2::on_nvim_call_atomic(QVariantList _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 156, _a);
}

// SIGNAL 157
void NeovimQt::NeovimApi2::err_nvim_call_atomic(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 157, _a);
}

// SIGNAL 158
void NeovimQt::NeovimApi2::on_nvim_win_get_buf(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 158, _a);
}

// SIGNAL 159
void NeovimQt::NeovimApi2::err_nvim_win_get_buf(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 159, _a);
}

// SIGNAL 160
void NeovimQt::NeovimApi2::on_nvim_win_get_cursor(QPoint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 160, _a);
}

// SIGNAL 161
void NeovimQt::NeovimApi2::err_nvim_win_get_cursor(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 161, _a);
}

// SIGNAL 162
void NeovimQt::NeovimApi2::on_nvim_win_set_cursor()
{
    QMetaObject::activate(this, &staticMetaObject, 162, nullptr);
}

// SIGNAL 163
void NeovimQt::NeovimApi2::err_nvim_win_set_cursor(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 163, _a);
}

// SIGNAL 164
void NeovimQt::NeovimApi2::on_nvim_win_get_height(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 164, _a);
}

// SIGNAL 165
void NeovimQt::NeovimApi2::err_nvim_win_get_height(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 165, _a);
}

// SIGNAL 166
void NeovimQt::NeovimApi2::on_nvim_win_set_height()
{
    QMetaObject::activate(this, &staticMetaObject, 166, nullptr);
}

// SIGNAL 167
void NeovimQt::NeovimApi2::err_nvim_win_set_height(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 167, _a);
}

// SIGNAL 168
void NeovimQt::NeovimApi2::on_nvim_win_get_width(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 168, _a);
}

// SIGNAL 169
void NeovimQt::NeovimApi2::err_nvim_win_get_width(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 169, _a);
}

// SIGNAL 170
void NeovimQt::NeovimApi2::on_nvim_win_set_width()
{
    QMetaObject::activate(this, &staticMetaObject, 170, nullptr);
}

// SIGNAL 171
void NeovimQt::NeovimApi2::err_nvim_win_set_width(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 171, _a);
}

// SIGNAL 172
void NeovimQt::NeovimApi2::on_nvim_win_get_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 172, _a);
}

// SIGNAL 173
void NeovimQt::NeovimApi2::err_nvim_win_get_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 173, _a);
}

// SIGNAL 174
void NeovimQt::NeovimApi2::on_nvim_win_set_var()
{
    QMetaObject::activate(this, &staticMetaObject, 174, nullptr);
}

// SIGNAL 175
void NeovimQt::NeovimApi2::err_nvim_win_set_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 175, _a);
}

// SIGNAL 176
void NeovimQt::NeovimApi2::on_nvim_win_del_var()
{
    QMetaObject::activate(this, &staticMetaObject, 176, nullptr);
}

// SIGNAL 177
void NeovimQt::NeovimApi2::err_nvim_win_del_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 177, _a);
}

// SIGNAL 178
void NeovimQt::NeovimApi2::on_window_set_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 178, _a);
}

// SIGNAL 179
void NeovimQt::NeovimApi2::err_window_set_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 179, _a);
}

// SIGNAL 180
void NeovimQt::NeovimApi2::on_window_del_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 180, _a);
}

// SIGNAL 181
void NeovimQt::NeovimApi2::err_window_del_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 181, _a);
}

// SIGNAL 182
void NeovimQt::NeovimApi2::on_nvim_win_get_option(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 182, _a);
}

// SIGNAL 183
void NeovimQt::NeovimApi2::err_nvim_win_get_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 183, _a);
}

// SIGNAL 184
void NeovimQt::NeovimApi2::on_nvim_win_set_option()
{
    QMetaObject::activate(this, &staticMetaObject, 184, nullptr);
}

// SIGNAL 185
void NeovimQt::NeovimApi2::err_nvim_win_set_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 185, _a);
}

// SIGNAL 186
void NeovimQt::NeovimApi2::on_nvim_win_get_position(QPoint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 186, _a);
}

// SIGNAL 187
void NeovimQt::NeovimApi2::err_nvim_win_get_position(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 187, _a);
}

// SIGNAL 188
void NeovimQt::NeovimApi2::on_nvim_win_get_tabpage(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 188, _a);
}

// SIGNAL 189
void NeovimQt::NeovimApi2::err_nvim_win_get_tabpage(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 189, _a);
}

// SIGNAL 190
void NeovimQt::NeovimApi2::on_nvim_win_get_number(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 190, _a);
}

// SIGNAL 191
void NeovimQt::NeovimApi2::err_nvim_win_get_number(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 191, _a);
}

// SIGNAL 192
void NeovimQt::NeovimApi2::on_nvim_win_is_valid(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 192, _a);
}

// SIGNAL 193
void NeovimQt::NeovimApi2::err_nvim_win_is_valid(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 193, _a);
}

// SIGNAL 194
void NeovimQt::NeovimApi2::on_buffer_line_count(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 194, _a);
}

// SIGNAL 195
void NeovimQt::NeovimApi2::err_buffer_line_count(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 195, _a);
}

// SIGNAL 196
void NeovimQt::NeovimApi2::on_buffer_get_lines(QList<QByteArray> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 196, _a);
}

// SIGNAL 197
void NeovimQt::NeovimApi2::err_buffer_get_lines(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 197, _a);
}

// SIGNAL 198
void NeovimQt::NeovimApi2::on_buffer_set_lines()
{
    QMetaObject::activate(this, &staticMetaObject, 198, nullptr);
}

// SIGNAL 199
void NeovimQt::NeovimApi2::err_buffer_set_lines(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 199, _a);
}

// SIGNAL 200
void NeovimQt::NeovimApi2::on_buffer_get_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 200, _a);
}

// SIGNAL 201
void NeovimQt::NeovimApi2::err_buffer_get_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 201, _a);
}

// SIGNAL 202
void NeovimQt::NeovimApi2::on_buffer_get_option(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 202, _a);
}

// SIGNAL 203
void NeovimQt::NeovimApi2::err_buffer_get_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 203, _a);
}

// SIGNAL 204
void NeovimQt::NeovimApi2::on_buffer_set_option()
{
    QMetaObject::activate(this, &staticMetaObject, 204, nullptr);
}

// SIGNAL 205
void NeovimQt::NeovimApi2::err_buffer_set_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 205, _a);
}

// SIGNAL 206
void NeovimQt::NeovimApi2::on_buffer_get_number(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 206, _a);
}

// SIGNAL 207
void NeovimQt::NeovimApi2::err_buffer_get_number(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 207, _a);
}

// SIGNAL 208
void NeovimQt::NeovimApi2::on_buffer_get_name(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 208, _a);
}

// SIGNAL 209
void NeovimQt::NeovimApi2::err_buffer_get_name(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 209, _a);
}

// SIGNAL 210
void NeovimQt::NeovimApi2::on_buffer_set_name()
{
    QMetaObject::activate(this, &staticMetaObject, 210, nullptr);
}

// SIGNAL 211
void NeovimQt::NeovimApi2::err_buffer_set_name(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 211, _a);
}

// SIGNAL 212
void NeovimQt::NeovimApi2::on_buffer_is_valid(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 212, _a);
}

// SIGNAL 213
void NeovimQt::NeovimApi2::err_buffer_is_valid(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 213, _a);
}

// SIGNAL 214
void NeovimQt::NeovimApi2::on_buffer_get_mark(QPoint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 214, _a);
}

// SIGNAL 215
void NeovimQt::NeovimApi2::err_buffer_get_mark(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 215, _a);
}

// SIGNAL 216
void NeovimQt::NeovimApi2::on_buffer_add_highlight(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 216, _a);
}

// SIGNAL 217
void NeovimQt::NeovimApi2::err_buffer_add_highlight(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 217, _a);
}

// SIGNAL 218
void NeovimQt::NeovimApi2::on_buffer_clear_highlight()
{
    QMetaObject::activate(this, &staticMetaObject, 218, nullptr);
}

// SIGNAL 219
void NeovimQt::NeovimApi2::err_buffer_clear_highlight(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 219, _a);
}

// SIGNAL 220
void NeovimQt::NeovimApi2::on_tabpage_get_windows(QList<int64_t> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 220, _a);
}

// SIGNAL 221
void NeovimQt::NeovimApi2::err_tabpage_get_windows(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 221, _a);
}

// SIGNAL 222
void NeovimQt::NeovimApi2::on_tabpage_get_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 222, _a);
}

// SIGNAL 223
void NeovimQt::NeovimApi2::err_tabpage_get_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 223, _a);
}

// SIGNAL 224
void NeovimQt::NeovimApi2::on_tabpage_get_window(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 224, _a);
}

// SIGNAL 225
void NeovimQt::NeovimApi2::err_tabpage_get_window(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 225, _a);
}

// SIGNAL 226
void NeovimQt::NeovimApi2::on_tabpage_is_valid(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 226, _a);
}

// SIGNAL 227
void NeovimQt::NeovimApi2::err_tabpage_is_valid(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 227, _a);
}

// SIGNAL 228
void NeovimQt::NeovimApi2::on_ui_detach()
{
    QMetaObject::activate(this, &staticMetaObject, 228, nullptr);
}

// SIGNAL 229
void NeovimQt::NeovimApi2::err_ui_detach(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 229, _a);
}

// SIGNAL 230
void NeovimQt::NeovimApi2::on_ui_try_resize(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 230, _a);
}

// SIGNAL 231
void NeovimQt::NeovimApi2::err_ui_try_resize(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 231, _a);
}

// SIGNAL 232
void NeovimQt::NeovimApi2::on_vim_command()
{
    QMetaObject::activate(this, &staticMetaObject, 232, nullptr);
}

// SIGNAL 233
void NeovimQt::NeovimApi2::err_vim_command(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 233, _a);
}

// SIGNAL 234
void NeovimQt::NeovimApi2::on_vim_feedkeys()
{
    QMetaObject::activate(this, &staticMetaObject, 234, nullptr);
}

// SIGNAL 235
void NeovimQt::NeovimApi2::err_vim_feedkeys(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 235, _a);
}

// SIGNAL 236
void NeovimQt::NeovimApi2::on_vim_input(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 236, _a);
}

// SIGNAL 237
void NeovimQt::NeovimApi2::err_vim_input(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 237, _a);
}

// SIGNAL 238
void NeovimQt::NeovimApi2::on_vim_replace_termcodes(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 238, _a);
}

// SIGNAL 239
void NeovimQt::NeovimApi2::err_vim_replace_termcodes(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 239, _a);
}

// SIGNAL 240
void NeovimQt::NeovimApi2::on_vim_command_output(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 240, _a);
}

// SIGNAL 241
void NeovimQt::NeovimApi2::err_vim_command_output(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 241, _a);
}

// SIGNAL 242
void NeovimQt::NeovimApi2::on_vim_eval(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 242, _a);
}

// SIGNAL 243
void NeovimQt::NeovimApi2::err_vim_eval(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 243, _a);
}

// SIGNAL 244
void NeovimQt::NeovimApi2::on_vim_call_function(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 244, _a);
}

// SIGNAL 245
void NeovimQt::NeovimApi2::err_vim_call_function(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 245, _a);
}

// SIGNAL 246
void NeovimQt::NeovimApi2::on_vim_strwidth(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 246, _a);
}

// SIGNAL 247
void NeovimQt::NeovimApi2::err_vim_strwidth(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 247, _a);
}

// SIGNAL 248
void NeovimQt::NeovimApi2::on_vim_list_runtime_paths(QList<QByteArray> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 248, _a);
}

// SIGNAL 249
void NeovimQt::NeovimApi2::err_vim_list_runtime_paths(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 249, _a);
}

// SIGNAL 250
void NeovimQt::NeovimApi2::on_vim_change_directory()
{
    QMetaObject::activate(this, &staticMetaObject, 250, nullptr);
}

// SIGNAL 251
void NeovimQt::NeovimApi2::err_vim_change_directory(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 251, _a);
}

// SIGNAL 252
void NeovimQt::NeovimApi2::on_vim_get_current_line(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 252, _a);
}

// SIGNAL 253
void NeovimQt::NeovimApi2::err_vim_get_current_line(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 253, _a);
}

// SIGNAL 254
void NeovimQt::NeovimApi2::on_vim_set_current_line()
{
    QMetaObject::activate(this, &staticMetaObject, 254, nullptr);
}

// SIGNAL 255
void NeovimQt::NeovimApi2::err_vim_set_current_line(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 255, _a);
}

// SIGNAL 256
void NeovimQt::NeovimApi2::on_vim_del_current_line()
{
    QMetaObject::activate(this, &staticMetaObject, 256, nullptr);
}

// SIGNAL 257
void NeovimQt::NeovimApi2::err_vim_del_current_line(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 257, _a);
}

// SIGNAL 258
void NeovimQt::NeovimApi2::on_vim_get_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 258, _a);
}

// SIGNAL 259
void NeovimQt::NeovimApi2::err_vim_get_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 259, _a);
}

// SIGNAL 260
void NeovimQt::NeovimApi2::on_vim_get_vvar(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 260, _a);
}

// SIGNAL 261
void NeovimQt::NeovimApi2::err_vim_get_vvar(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 261, _a);
}

// SIGNAL 262
void NeovimQt::NeovimApi2::on_vim_get_option(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 262, _a);
}

// SIGNAL 263
void NeovimQt::NeovimApi2::err_vim_get_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 263, _a);
}

// SIGNAL 264
void NeovimQt::NeovimApi2::on_vim_set_option()
{
    QMetaObject::activate(this, &staticMetaObject, 264, nullptr);
}

// SIGNAL 265
void NeovimQt::NeovimApi2::err_vim_set_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 265, _a);
}

// SIGNAL 266
void NeovimQt::NeovimApi2::on_vim_out_write()
{
    QMetaObject::activate(this, &staticMetaObject, 266, nullptr);
}

// SIGNAL 267
void NeovimQt::NeovimApi2::err_vim_out_write(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 267, _a);
}

// SIGNAL 268
void NeovimQt::NeovimApi2::on_vim_err_write()
{
    QMetaObject::activate(this, &staticMetaObject, 268, nullptr);
}

// SIGNAL 269
void NeovimQt::NeovimApi2::err_vim_err_write(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 269, _a);
}

// SIGNAL 270
void NeovimQt::NeovimApi2::on_vim_report_error()
{
    QMetaObject::activate(this, &staticMetaObject, 270, nullptr);
}

// SIGNAL 271
void NeovimQt::NeovimApi2::err_vim_report_error(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 271, _a);
}

// SIGNAL 272
void NeovimQt::NeovimApi2::on_vim_get_buffers(QList<int64_t> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 272, _a);
}

// SIGNAL 273
void NeovimQt::NeovimApi2::err_vim_get_buffers(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 273, _a);
}

// SIGNAL 274
void NeovimQt::NeovimApi2::on_vim_get_current_buffer(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 274, _a);
}

// SIGNAL 275
void NeovimQt::NeovimApi2::err_vim_get_current_buffer(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 275, _a);
}

// SIGNAL 276
void NeovimQt::NeovimApi2::on_vim_set_current_buffer()
{
    QMetaObject::activate(this, &staticMetaObject, 276, nullptr);
}

// SIGNAL 277
void NeovimQt::NeovimApi2::err_vim_set_current_buffer(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 277, _a);
}

// SIGNAL 278
void NeovimQt::NeovimApi2::on_vim_get_windows(QList<int64_t> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 278, _a);
}

// SIGNAL 279
void NeovimQt::NeovimApi2::err_vim_get_windows(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 279, _a);
}

// SIGNAL 280
void NeovimQt::NeovimApi2::on_vim_get_current_window(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 280, _a);
}

// SIGNAL 281
void NeovimQt::NeovimApi2::err_vim_get_current_window(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 281, _a);
}

// SIGNAL 282
void NeovimQt::NeovimApi2::on_vim_set_current_window()
{
    QMetaObject::activate(this, &staticMetaObject, 282, nullptr);
}

// SIGNAL 283
void NeovimQt::NeovimApi2::err_vim_set_current_window(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 283, _a);
}

// SIGNAL 284
void NeovimQt::NeovimApi2::on_vim_get_tabpages(QList<int64_t> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 284, _a);
}

// SIGNAL 285
void NeovimQt::NeovimApi2::err_vim_get_tabpages(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 285, _a);
}

// SIGNAL 286
void NeovimQt::NeovimApi2::on_vim_get_current_tabpage(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 286, _a);
}

// SIGNAL 287
void NeovimQt::NeovimApi2::err_vim_get_current_tabpage(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 287, _a);
}

// SIGNAL 288
void NeovimQt::NeovimApi2::on_vim_set_current_tabpage()
{
    QMetaObject::activate(this, &staticMetaObject, 288, nullptr);
}

// SIGNAL 289
void NeovimQt::NeovimApi2::err_vim_set_current_tabpage(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 289, _a);
}

// SIGNAL 290
void NeovimQt::NeovimApi2::on_vim_subscribe()
{
    QMetaObject::activate(this, &staticMetaObject, 290, nullptr);
}

// SIGNAL 291
void NeovimQt::NeovimApi2::err_vim_subscribe(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 291, _a);
}

// SIGNAL 292
void NeovimQt::NeovimApi2::on_vim_unsubscribe()
{
    QMetaObject::activate(this, &staticMetaObject, 292, nullptr);
}

// SIGNAL 293
void NeovimQt::NeovimApi2::err_vim_unsubscribe(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 293, _a);
}

// SIGNAL 294
void NeovimQt::NeovimApi2::on_vim_name_to_color(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 294, _a);
}

// SIGNAL 295
void NeovimQt::NeovimApi2::err_vim_name_to_color(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 295, _a);
}

// SIGNAL 296
void NeovimQt::NeovimApi2::on_vim_get_color_map(QVariantMap _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 296, _a);
}

// SIGNAL 297
void NeovimQt::NeovimApi2::err_vim_get_color_map(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 297, _a);
}

// SIGNAL 298
void NeovimQt::NeovimApi2::on_window_get_buffer(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 298, _a);
}

// SIGNAL 299
void NeovimQt::NeovimApi2::err_window_get_buffer(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 299, _a);
}

// SIGNAL 300
void NeovimQt::NeovimApi2::on_window_get_cursor(QPoint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 300, _a);
}

// SIGNAL 301
void NeovimQt::NeovimApi2::err_window_get_cursor(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 301, _a);
}

// SIGNAL 302
void NeovimQt::NeovimApi2::on_window_set_cursor()
{
    QMetaObject::activate(this, &staticMetaObject, 302, nullptr);
}

// SIGNAL 303
void NeovimQt::NeovimApi2::err_window_set_cursor(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 303, _a);
}

// SIGNAL 304
void NeovimQt::NeovimApi2::on_window_get_height(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 304, _a);
}

// SIGNAL 305
void NeovimQt::NeovimApi2::err_window_get_height(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 305, _a);
}

// SIGNAL 306
void NeovimQt::NeovimApi2::on_window_set_height()
{
    QMetaObject::activate(this, &staticMetaObject, 306, nullptr);
}

// SIGNAL 307
void NeovimQt::NeovimApi2::err_window_set_height(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 307, _a);
}

// SIGNAL 308
void NeovimQt::NeovimApi2::on_window_get_width(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 308, _a);
}

// SIGNAL 309
void NeovimQt::NeovimApi2::err_window_get_width(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 309, _a);
}

// SIGNAL 310
void NeovimQt::NeovimApi2::on_window_set_width()
{
    QMetaObject::activate(this, &staticMetaObject, 310, nullptr);
}

// SIGNAL 311
void NeovimQt::NeovimApi2::err_window_set_width(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 311, _a);
}

// SIGNAL 312
void NeovimQt::NeovimApi2::on_window_get_var(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 312, _a);
}

// SIGNAL 313
void NeovimQt::NeovimApi2::err_window_get_var(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 313, _a);
}

// SIGNAL 314
void NeovimQt::NeovimApi2::on_window_get_option(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 314, _a);
}

// SIGNAL 315
void NeovimQt::NeovimApi2::err_window_get_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 315, _a);
}

// SIGNAL 316
void NeovimQt::NeovimApi2::on_window_set_option()
{
    QMetaObject::activate(this, &staticMetaObject, 316, nullptr);
}

// SIGNAL 317
void NeovimQt::NeovimApi2::err_window_set_option(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 317, _a);
}

// SIGNAL 318
void NeovimQt::NeovimApi2::on_window_get_position(QPoint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 318, _a);
}

// SIGNAL 319
void NeovimQt::NeovimApi2::err_window_get_position(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 319, _a);
}

// SIGNAL 320
void NeovimQt::NeovimApi2::on_window_get_tabpage(int64_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 320, _a);
}

// SIGNAL 321
void NeovimQt::NeovimApi2::err_window_get_tabpage(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 321, _a);
}

// SIGNAL 322
void NeovimQt::NeovimApi2::on_window_is_valid(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 322, _a);
}

// SIGNAL 323
void NeovimQt::NeovimApi2::err_window_is_valid(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 323, _a);
}
QT_WARNING_POP
